/*
 * arch/ppc/platforms/apollo_pci.c
 * 
 * PCI setup routines for Accton Apollo boards.
 *
 * Author: John W. Linville
 *         linville@lvl7.com
 *
 * Copyright 2002 LVL7 Systems Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/pci.h>
#include <linux/slab.h>

#include <asm/byteorder.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/machdep.h>
#include <asm/mpc10x.h>
#include <asm/pci-bridge.h>

#include "apollo.h"

/*
 * Accton Apollo interrupt routes.
 */
static inline int
apollo_map_irq(struct pci_dev *dev, unsigned char idsel, unsigned char pin)
{
        static char pci_irq_table[][4] =
        /*
         *      PCI IDSEL/INTPIN->INTLINE
         *         A   B   C   D
         */
        {
                { 16,  0,  0,  0 }     /* IDSEL 19 - IXE2424 */
        };

        const long min_idsel = 19, max_idsel = 19, irqs_per_slot = 4;
        return PCI_IRQ_TABLE_LOOKUP;
}

void __init
apollo_find_bridges(void)
{
	struct pci_controller	*hose;

	hose = pcibios_alloc_controller();

	if (!hose)
		return;

	hose->first_busno = 0;
	hose->last_busno = 0xff;

	if (mpc10x_bridge_init(hose,
			       MPC10X_MEM_MAP_B,
			       MPC10X_MEM_MAP_B,
			       APOLLO_EUMB_BASE) == 0) {

		hose->mem_resources[0].end = 0xffffffff;
	
		/* scan PCI bus */
		ppc_md.pci_exclude_device = NULL;
		hose->last_busno = pciauto_bus_scan(hose, hose->first_busno);

		ppc_md.pcibios_fixup = NULL;
		ppc_md.pcibios_fixup_bus = NULL;
		ppc_md.pci_swizzle = common_swizzle;
		ppc_md.pci_map_irq = apollo_map_irq;
	}
	else {
		if (ppc_md.progress)
			ppc_md.progress("Bridge init failed", 0x100);
		printk("Host bridge init failed\n");
	}

	return;
}
