/*
 * BK Id: %F% %I% %G% %U% %#%
 *
 *    Copyright 2001 MontaVista Software Inc.
 *        <mlocke@mvista.com>
 *
 * 	Not much needed for the Embedded Planet 405gp board
 *
 *  	History: 11/09/2001 - armin
 *      added board_init to add in additional instuctions needed during platfrom_init
 *	cleaned up map_irq.
 *
 */
#include <linux/config.h>
#include <linux/init.h>
#include <linux/pci.h>
#include <asm/system.h>
#include <asm/pci-bridge.h>
#include <asm/machdep.h>
#include <asm/todc.h>

#undef DEBUG
#ifdef DEBUG
#define DBG(x...) printk(x)
#else
#define DBG(x...)
#endif

void *ep405_bcsr;
void *ep405_nvram;

int __init
ppc405_map_irq(struct pci_dev *dev, unsigned char idsel, unsigned char pin)
{
	static char pci_irq_table[][4] =
	/*
	 *      PCI IDSEL/INTPIN->INTLINE
	 *      A       B       C       D
	 */
	{
                {28, 28, 28, 28},       /* IDSEL 1 - PCI slot 1 */
                {29, 29, 29, 29},       /* IDSEL 2 - PCI slot 2 */
                {30, 30, 30, 30},       /* IDSEL 3 - PCI slot 3 */
                {31, 31, 31, 31},       /* IDSEL 4 - PCI slot 4 */
        };
	const long min_idsel = 1, max_idsel = 4, irqs_per_slot = 4;
	return PCI_IRQ_TABLE_LOOKUP;
};


void __init
board_setup_arch(void)
{
#ifdef CONFIG_PPC_RTC
	/* FIXME: what if NVRAM size is not 512k */
	TODC_INIT(TODC_TYPE_DS1557, ep405_nvram, ep405_nvram, ep405_nvram, 8);
#endif /* CONFIG_PPC_RTC */
}

void __init
bios_fixup(struct pci_controller *hose, void *pcil0_base)
{

	unsigned int bar_response, bar;

	/*
	** Expected PCI mapping:
	**
	**  PLB addr		    PCI memory addr
	**  ---------------------       ---------------------
	**  0000'0000 - 7fff'ffff <---  0000'0000 - 7fff'ffff
	**  8000'0000 - Bfff'ffff --->  8000'0000 - Bfff'ffff
	**
	**  PLB addr		    PCI io addr
	**  ---------------------       ---------------------
	**  e800'0000 - e800'ffff --->  0000'0000 - 0001'0000
	**
	*/

#ifdef DEBUG
	int i;
	printk("ioremap PCLIO_BASE = 0x%x\n",pcil0_base);
	for (i = PCIL0_PMM0LA; i <= PCIL0_PTM2LA; i+=4)
		printk( "PCI bridge before reg %x : 0x%x\n", i,
				in_le32(pcil0_base + i));
#endif

	/* Disable region first */
	out_le32(pcil0_base + PCIL0_PMM0MA, 0x00000000);
	/* PLB starting addr, PCI: 0x80000000 */
	out_le32(pcil0_base + PCIL0_PMM0LA, 0x80000000);
	/* PCI start addr, 0x80000000 */
	out_le32(pcil0_base + PCIL0_PMM0PCILA, PPC405_PCI_MEM_BASE);
	/* 512MB range of PLB to PCI */
	out_le32(pcil0_base + PCIL0_PMM0PCIHA, 0x00000000);
	/* Enable no pre-fetch, enable region */
	out_le32(pcil0_base + PCIL0_PMM0MA, ((0xffffffff -
	(PPC405_PCI_UPPER_MEM - PPC405_PCI_MEM_BASE)) | 0x01 ));

	/* Disable region one */
	out_le32(pcil0_base + PCIL0_PMM1MA, 0x00000000);
	out_le32(pcil0_base + PCIL0_PMM1LA, 0x00000000);
	out_le32(pcil0_base + PCIL0_PMM1PCILA, 0x00000000);
	out_le32(pcil0_base + PCIL0_PMM1PCIHA, 0x00000000);
	out_le32(pcil0_base + PCIL0_PMM1MA, 0x00000000);
	out_le32(pcil0_base + PCIL0_PTM1MS, 0x00000000);

	/* Disable region two */
	out_le32(pcil0_base + PCIL0_PMM2MA, 0x00000000);
	out_le32(pcil0_base + PCIL0_PMM2LA, 0x00000000);
	out_le32(pcil0_base + PCIL0_PMM2PCILA, 0x00000000);
	out_le32(pcil0_base + PCIL0_PMM2PCIHA, 0x00000000);
	out_le32(pcil0_base + PCIL0_PTM2MS,0x00000000);

	/* Zero config bars */
	for (bar= PCI_BASE_ADDRESS_1; bar <= PCI_BASE_ADDRESS_2; bar+=4) {
		early_write_config_dword(hose, hose->first_busno,
				PCI_FUNC(hose->first_busno), bar, 0x00000000);
		early_read_config_dword(hose, hose->first_busno,
				PCI_FUNC(hose->first_busno), bar,
				&bar_response);
		DBG("BUS %d, device %d, Function %d bar 0x%8.8x is 0x%8.8x\n",
				hose->first_busno, PCI_SLOT( hose->first_busno),
				PCI_FUNC(hose->first_busno),bar,bar_response);
	}
	/* end work arround */

#ifdef DEBUG
	for (i = PCIL0_PMM0LA;i <= PCIL0_PTM2LA; i+=4)
		printk( "PCI bridge after reg %x : 0x%x\n", i,
				in_le32(pcil0_base + i));
#endif
}

void __init
board_io_mapping(void)
{
	ep405_bcsr = ioremap(EP405_BCSR_PADDR, EP405_BCSR_SIZE);
	ep405_nvram = ioremap(EP405_NVRAM_PADDR, EP405_NVRAM_SIZE);
}

void __init
board_setup_irq(void)
{
}

void __init
board_init(void)
{

#ifdef CONFIG_PPC_RTC
	ppc_md.time_init = todc_time_init;
	ppc_md.set_rtc_time = todc_set_rtc_time;
	ppc_md.get_rtc_time = todc_get_rtc_time;
	ppc_md.nvram_read_val = todc_direct_read_val;
	ppc_md.nvram_write_val = todc_direct_write_val;
#endif

}
