/*
 * arch/ppc/platforms/mousse_pci.c
 * 
 * PCI setup routines for the Mouse board on the Broadcom Reference Platform
 *
 * Author: John W. Linville
 *         linville@lvl7.com
 *
 * Copyright 2002 LVL7 Systems Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/pci.h>
#include <linux/slab.h>

#include <asm/byteorder.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/uaccess.h>
#include <asm/machdep.h>
#include <asm/pci-bridge.h>
#include <asm/mpc10x.h>

#include "mousse.h"

/*
 * Mousse Interrupt Mapping:
 *
 *      IRQ1    Enet (intA|intB|intC|intD)
 *      IRQ2    CPCI intA (See below)
 *      IRQ3    Local PCI slot intA|intB|intC|intD
 *      IRQ4    COM1 Serial port (Actually higher addressed port on duart)
 *
 * PCI Interrupt Mapping in CPCI Backplane:
 *
 *                 |           CPCI Slot
 *                 | 1 (CPU)    2       3       4       5       6
 *      -----------+--------+-------+-------+-------+-------+-------+
 *        intA     |    X               X               X
 *        intB     |            X               X               X
 *        intC     |    X               X               X
 *        intD     |            X               X               X
 */
static inline int
mousse_map_irq(struct pci_dev *dev, unsigned char idsel, unsigned char pin)
{
	static char pci_irq_table[][4] =
	/*
	 *	PCI IDSEL/INTPIN->INTLINE 
	 * 	   A   B   C   D
	 */
	{
		{ 17, 17, 17, 17 },	/* IDSEL 13 - DEC 21143 Enet */
		{ 19, 19, 19, 19 },	/* IDSEL 14 - Onboard PCI */
		{  0,  0,  0,  0 },	/* IDSEL 15 - Intel 82371 */
		{  0,  0,  0,  0 },	/* IDSEL 16 - unused */
		{  0,  0,  0,  0 },	/* IDSEL 17 - unused */
		{  0,  0,  0,  0 },	/* IDSEL 18 - unused */
		{  0,  0,  0,  0 },	/* IDSEL 19 - unused */
		{  0,  0,  0,  0 },	/* IDSEL 20 - unused */
		{  0,  0,  0,  0 },	/* IDSEL 21 - unused */
		{  0,  0,  0,  0 },	/* IDSEL 22 - unused */
		{  0,  0,  0,  0 },	/* IDSEL 23 - unused */
		{  0,  0,  0,  0 },	/* IDSEL 24 - unused */
		{  0,  0,  0,  0 },	/* IDSEL 25 - unused */
		{  0,  0,  0,  0 },	/* IDSEL 26 - unused */
		{  0, 18,  0, 18 },	/* IDSEL 27 - CPCI Slot 6 */
		{ 18,  0, 18,  0 },	/* IDSEL 28 - CPCI Slot 5 */
		{  0, 18,  0, 18 },	/* IDSEL 29 - CPCI Slot 4 */
		{ 18,  0, 18,  0 },	/* IDSEL 30 - CPCI Slot 3 */
		{  0, 18,  0, 18 },	/* IDSEL 31 - CPCI Slot 2 */
	};

	const long min_idsel = 13, max_idsel = 31, irqs_per_slot = 4;
	return PCI_IRQ_TABLE_LOOKUP;
}

void __init
mousse_find_bridges(void)
{
	struct pci_controller	*hose;

	hose = pcibios_alloc_controller();

	if (!hose)
		return;

	hose->first_busno = 0;
	hose->last_busno = 0xff;

	if (mpc10x_bridge_init(hose,
			       MPC10X_MEM_MAP_B,
			       MPC10X_MEM_MAP_B,
			       MPC10X_MAPB_EUMB_BASE) == 0) {

		hose->mem_resources[0].end = 0xffffffff;

		/* Scan PCI bus... */
		hose->last_busno = pciauto_bus_scan(hose, hose->first_busno);

		ppc_md.pcibios_fixup = NULL;
		ppc_md.pcibios_fixup_bus = NULL;
		ppc_md.pci_swizzle = common_swizzle;
		ppc_md.pci_map_irq = mousse_map_irq;
	}
	else {
		if (ppc_md.progress)
			ppc_md.progress("Bridge init failed", 0x100);
		printk("Host bridge init failed\n");
	}

	return;
}
