/*
 * arch/ppc/platforms/psg_setup.c
 *
 * Board setup routines for Jabil PSG board.
 *
 * Author: John W. Linville
 *         linville@lvl7.com
 *
 * Copyright 2003 LVL7 Systems Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/config.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/reboot.h>
#include <linux/pci.h>
#include <linux/kdev_t.h>
#include <linux/types.h>
#include <linux/major.h>
#include <linux/blk.h>
#include <linux/console.h>
#include <linux/delay.h>
#include <linux/irq.h>
#include <linux/seq_file.h>

#include <asm/system.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/dma.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/prom.h>
#include <asm/time.h>
#include <asm/todc.h>
#include <asm/open_pic.h>
#include <asm/mpc10x.h>
#include <asm/pci-bridge.h>
#include <asm/bootinfo.h>

#include "psg.h"

unsigned char __res[sizeof(bd_t)];

static u_char psg_openpic_initsenses[] __initdata = {
	0,	/* 0-15 are not used on an 8240 EPIC */
	0,	/* 1 */
	0,	/* 2 */
	0,	/* 3 */
	0,	/* 4 */
	0,	/* 5 */
	0,	/* 6 */
	0,	/* 7 */
	0,	/* 8 */
	0,	/* 9 */
	0,	/* 10 */
	0,	/* 11 */
	0,	/* 12 */
	0,	/* 13 */
	0,	/* 14 */
	0,	/* 15 */
	1,	/* 16: EPIC SINT 0/IRQ 0: Active Low -- BCM5690 #1 */
	1,	/* 17: EPIC SINT 1/IRQ 1: Active Low -- BCM5690 #2 */
	1,	/* 18: EPIC SINT 2/IRQ 2: Active Low -- unused     */
	1,	/* 19: EPIC SINT 3/IRQ 3: Active Low -- unused     */
        1,      /* 20: EPIC SINT 4/IRQ 4: Active Low -- unused     */
        0,      /* 21: EPIC SINT 5: Active Low -- unused */
        0,      /* 22: EPIC SINT 6: Active Low -- unused */
        0,      /* 23: EPIC SINT 7: Active Low -- unused */
        0,      /* 24: EPIC SINT 8: Active Low -- unused */
        0,      /* 25: EPIC SINT 9: Active Low -- unused */
        0,      /* 26: EPIC SINT 10: Active Low -- unused */
        0,      /* 27: EPIC SINT 11: Active Low -- unused */
        0,      /* 28: EPIC SINT 12: Active Low -- unused */
        0,      /* 29: EPIC SINT 13: Active Low -- unused */
        0,      /* 30: EPIC SINT 14: Active Low -- unused */
        0,      /* 31: EPIC SINT 15: Active Low -- unused */
        0,      /* 32: EPIC Reserved (0x400): Active Low -- unused */
        0,      /* 33: EPIC Reserved (0x420): Active Low -- unused */
        0,      /* 34: EPIC Reserved (0x440): Active Low -- unused */
        0,      /* 35: EPIC Reserved (0x460): Active Low -- unused */
        0,      /* 36: EPIC Reserved (0x480): Active Low -- unused */
        0,      /* 37: EPIC Reserved (0x4a0): Active Low -- unused */
        0,      /* 38: EPIC Reserved (0x4c0): Active Low -- unused */
        0,      /* 39: EPIC Reserved (0x4e0): Active Low -- unused */
        0,      /* 40: EPIC Reserved (0x500): Active Low -- unused */
        0,      /* 41: EPIC Reserved (0x520): Active Low -- unused */
        0,      /* 42: EPIC Reserved (0x540): Active Low -- unused */
        0,      /* 43: EPIC Reserved (0x560): Active Low -- unused */
        0,      /* 44: EPIC Reserved (0x580): Active Low -- unused */
        0,      /* 45: EPIC Reserved (0x5a0): Active Low -- unused */
        0,      /* 46: EPIC Reserved (0x5c0): Active Low -- unused */
        0,      /* 47: EPIC Reserved (0x5e0): Active Low -- unused */
        0,      /* 48: EPIC Reserved (0x600): Active Low -- unused */
        0,      /* 49: EPIC Reserved (0x620): Active Low -- unused */
        0,      /* 50: EPIC Reserved (0x640): Active Low -- unused */
        0,      /* 51: EPIC Reserved (0x660): Active Low -- unused */
        0,      /* 52: EPIC Reserved (0x680): Active Low -- unused */
        0,      /* 53: EPIC Reserved (0x6a0): Active Low -- unused */
        0,      /* 54: EPIC Reserved (0x6c0): Active Low -- unused */
        0,      /* 55: EPIC Reserved (0x6e0): Active Low -- unused */
        0,      /* 56: EPIC Reserved (0x700): Active Low -- unused */
        0,      /* 57: EPIC Reserved (0x720): Active Low -- unused */
        0,      /* 58: EPIC Reserved (0x740): Active Low -- unused */
        0,      /* 59: EPIC Reserved (0x760): Active Low -- unused */
        0,      /* 60: EPIC Reserved (0x780): Active Low -- unused */
        0,      /* 61: EPIC Reserved (0x7a0): Active Low -- unused */
        0,      /* 62: EPIC Reserved (0x7c0): Active Low -- unused */
        0,      /* 63: EPIC Reserved (0x7e0): Active Low -- unused */
        0,      /* 64: EPIC Reserved (0x800): Active Low -- unused */
        0,      /* 65: EPIC Reserved (0x820): Active Low -- unused */
        0,      /* 66: EPIC Reserved (0x840): Active Low -- unused */
        0,      /* 67: EPIC Reserved (0x860): Active Low -- unused */
        0,      /* 68: EPIC Reserved (0x880): Active Low -- unused */
        0,      /* 69: EPIC Reserved (0x8a0): Active Low -- unused */
        0,      /* 70: EPIC Reserved (0x8c0): Active Low -- unused */
        0,      /* 71: EPIC Reserved (0x8e0): Active Low -- unused */
        0,      /* 72: EPIC Reserved (0x900): Active Low -- unused */
        0,      /* 73: EPIC Reserved (0x920): Active Low -- unused */
        0,      /* 74: EPIC Reserved (0x940): Active Low -- unused */
        0,      /* 75: EPIC Reserved (0x960): Active Low -- unused */
        0,      /* 76: EPIC Reserved (0x980): Active Low -- unused */
        0,      /* 77: EPIC Reserved (0x9a0): Active Low -- unused */
        0,      /* 78: EPIC Reserved (0x9c0): Active Low -- unused */
        0,      /* 79: EPIC Reserved (0x9e0): Active Low -- unused */
        0,      /* 80: EPIC Reserved (0xa00): Active Low -- unused */
        0,      /* 81: EPIC Reserved (0xa20): Active Low -- unused */
        0,      /* 82: EPIC Reserved (0xa40): Active Low -- unused */
        0,      /* 83: EPIC Reserved (0xa60): Active Low -- unused */
        0,      /* 84: EPIC Reserved (0xa80): Active Low -- unused */
        0,      /* 85: EPIC Reserved (0xaa0): Active Low -- unused */
        0,      /* 86: EPIC Reserved (0xac0): Active Low -- unused */
        0,      /* 87: EPIC Reserved (0xae0): Active Low -- unused */
        0,      /* 88: EPIC Reserved (0xb00): Active Low -- unused */
        0,      /* 89: EPIC Reserved (0xb20): Active Low -- unused */
        0,      /* 90: EPIC Reserved (0xb40): Active Low -- unused */
        0,      /* 91: EPIC Reserved (0xb60): Active Low -- unused */
        0,      /* 92: EPIC Reserved (0xb80): Active Low -- unused */
        0,      /* 93: EPIC Reserved (0xba0): Active Low -- unused */
        0,      /* 94: EPIC Reserved (0xbc0): Active Low -- unused */
        0,      /* 95: EPIC Reserved (0xbe0): Active Low -- unused */
        0,      /* 96: EPIC Reserved (0xc00): Active Low -- unused */
        0,      /* 97: EPIC Reserved (0xc20): Active Low -- unused */
        0,      /* 98: EPIC Reserved (0xc40): Active Low -- unused */
        0,      /* 99: EPIC Reserved (0xc60): Active Low -- unused */
        0,      /* 100: EPIC Reserved (0xc80): Active Low -- unused */
        0,      /* 101: EPIC Reserved (0xca0): Active Low -- unused */
        0,      /* 102: EPIC Reserved (0xcc0): Active Low -- unused */
        0,      /* 103: EPIC Reserved (0xce0): Active Low -- unused */
        0,      /* 104: EPIC Reserved (0xd00): Active Low -- unused */
        0,      /* 105: EPIC Reserved (0xd20): Active Low -- unused */
        0,      /* 106: EPIC Reserved (0xd40): Active Low -- unused */
        0,      /* 107: EPIC Reserved (0xd60): Active Low -- unused */
        0,      /* 108: EPIC Reserved (0xd80): Active Low -- unused */
        0,      /* 109: EPIC Reserved (0xda0): Active Low -- unused */
        0,      /* 110: EPIC Reserved (0xdc0): Active Low -- unused */
        0,      /* 111: EPIC Reserved (0xde0): Active Low -- unused */
        0,      /* 112: EPIC Reserved (0xe00): Active Low -- unused */
        0,      /* 113: EPIC Reserved (0xe20): Active Low -- unused */
        0,      /* 114: EPIC Reserved (0xe40): Active Low -- unused */
        0,      /* 115: EPIC Reserved (0xe60): Active Low -- unused */
        0,      /* 116: EPIC Reserved (0xe80): Active Low -- unused */
        0,      /* 117: EPIC Reserved (0xea0): Active Low -- unused */
        0,      /* 118: EPIC Reserved (0xec0): Active Low -- unused */
        0,      /* 119: EPIC Reserved (0xee0): Active Low -- unused */
        0,      /* 120: EPIC Reserved (0xf00): Active Low -- unused */
        0,      /* 121: EPIC Reserved (0xf20): Active Low -- unused */
        0,      /* 122: EPIC Reserved (0xf40): Active Low -- unused */
        0,      /* 123: EPIC Reserved (0xf60): Active Low -- unused */
        0,      /* 124: EPIC Reserved (0xf80): Active Low -- unused */
        0,      /* 125: EPIC Reserved (0xfa0): Active Low -- unused */
        0,      /* 126: EPIC Reserved (0xfc0): Active Low -- unused */
        0,      /* 127: EPIC Reserved (0xfe0): Active Low -- unused */
        0,      /* 128: EPIC Reserved (0x1000): Active Low -- unused */
        1,      /* 129: EPIC I2C: Active Low -- I2C interface  */
        1,      /* 130: EPIC DMA Ch 0: Active Low -- unused */
        1,      /* 131: EPIC DMA Ch 1: Active Low -- unused */
        0,      /* 132: EPIC Reserved (0x1080): Active Low -- unused */
        0,      /* 133: EPIC Reserved (0x10a0): Active Low -- unused */
        1,      /* 134: EPIC Message Unit: Active Low -- unused */
        0,      /* 135: EPIC Reserved (0x10e0): Active Low -- unused */
        0,      /* 136: EPIC Reserved (0x1100): Active Low -- unused */
        1,      /* 137: EPIC DUART Ch 1: Active Low -- console port */
        1,      /* 138: EPIC DUART Ch 2: Active Low -- unused (?) */
};


static void __init
psg_setup_arch(void)
{
	loops_per_jiffy = 500000000 / HZ;

#ifdef CONFIG_BLK_DEV_INITRD
	if (initrd_start)
		ROOT_DEV = MKDEV(RAMDISK_MAJOR, 0);
#endif
#if defined(CONFIG_BLK_DEV_INITRD) && defined(CONFIG_ROOT_NFS)
	else
#endif
#ifdef CONFIG_ROOT_NFS
		ROOT_DEV = to_kdev_t(0x00FF);   /* /dev/nfs pseudo device */
#endif

	printk("Jabil PSG 24 Port 10/100/1000 Ethernet Switch\n");

	/* Lookup PCI host bridge */
	psg_find_bridges();

	return;
}

static ulong __init
psg_find_end_of_memory(void)
{
	bd_t *binfo;
	extern unsigned char __res[];

	binfo = (bd_t *)__res;

	return binfo->bi_memsize;
}

static void __init
psg_map_io(void)
{
	io_block_mapping(0xfc000000, 0xfc000000, 0x00100000, _PAGE_IO);
	io_block_mapping(0xff000000, 0xff000000, 0x01000000, _PAGE_IO);
	io_block_mapping(PSG_CPLD_VBASE, PSG_CPLD_PBASE, PSG_CPLD_SIZE, _PAGE_IO);
}

static void __init
psg_init_IRQ(void)
{
	OpenPIC_InitSenses = psg_openpic_initsenses;
	OpenPIC_NumInitSenses = sizeof(psg_openpic_initsenses);

	openpic_init(1, 0, NULL, -1);

	return;
}

static void
psg_restart(char *cmd)
{
	__cli();

	out_8((volatile u_char *)PSG_CPLD_RESET_ADDR, PSG_CPLD_RESET_VAL);
	for (;;);

	panic("Restart failed.\n");
	/* NOTREACHED */
}

static void
psg_power_off(void)
{
	__cli();
	for(;;);  /* No way to shut power off with software */
	/* NOTREACHED */
}

static void
psg_halt(void)
{
	psg_power_off();
	/* NOTREACHED */
}

static int
psg_show_cpuinfo(struct seq_file *m)
{
	bd_t *bp;

	bp = (bd_t *)__res;

	seq_printf(m, "vendor\t\t: Jabil\n");
	seq_printf(m, "machine\t\t: PSG\n");
	seq_printf(m, "cpu speed\t: %ldMhz\n",
			bp->bi_intfreq/1000000);
	seq_printf(m, "bus speed\t: %ldMhz\n",
			bp->bi_busfreq/1000000);

	return 0;
}

static void __init
psg_calibrate_decr(void)
{
	bd_t    *binfo = (bd_t *)__res;
	ulong freq;

	freq = (binfo->bi_busfreq / 4);

	printk("time_init: decrementer frequency = %lu.%.6lu MHz\n",
	       freq/1000000, freq%1000000);

	tb_ticks_per_jiffy = freq / HZ;
	tb_to_us = mulhwu_scale_factor(freq, 1000000);
#ifdef CONFIG_ILATENCY
	{
		extern unsigned ticks_per_usec;
		ticks_per_usec = freq / 1000000;
	}
#endif

	return;
}

unsigned char
nvram_read_byte(int i)
{
	if (ppc_md.nvram_read_val)
		return ppc_md.nvram_read_val(i);

	return 0;
}

void
nvram_write_byte(unsigned char c, int i)
{
	if (ppc_md.nvram_write_val)
		return ppc_md.nvram_write_val(i, c);

	return;
}

/*
 * Set BAT 3 to map 0xfc000000 to end of physical memory space 1-to-1.
 */
static __inline__ void
psg_set_bat(void)
{
	unsigned long   bat3u, bat3l; 
	static int	mapping_set = 0;

	if (!mapping_set) {
		__asm__ __volatile__(
		" lis %0,0xfc00\n \
		  ori %1,%0,0x002a\n \
		  ori %0,%0,0x07fe\n \
		  mtspr 0x21e,%0\n \
		  mtspr 0x21f,%1\n \
		  isync\n \
		  sync "
		  : "=r" (bat3u), "=r" (bat3l));

		mapping_set = 1;
	}

	return;
}

#ifdef	CONFIG_SERIAL_TEXT_DEBUG
#include <linux/serialP.h>
#include <linux/serial_reg.h>
#include <asm/serial.h>

static struct serial_state rs_table[RS_TABLE_SIZE] = {
	SERIAL_PORT_DFNS	/* Defined in <asm/serial.h> */
};

void
psg_progress(char *s, unsigned short hex)
{
	volatile char c;
	volatile unsigned long com_port;
	u16 shift;

	com_port = rs_table[0].port;
	shift = rs_table[0].iomem_reg_shift;

	while ((c = *s++) != 0) {
		while ((*((volatile unsigned char *)com_port +
				(UART_LSR << shift)) & UART_LSR_THRE) == 0)
		                ;
	        *(volatile unsigned char *)com_port = c;

		if (c == '\n') {
			while ((*((volatile unsigned char *)com_port +
				(UART_LSR << shift)) & UART_LSR_THRE) == 0)
					;
	        	*(volatile unsigned char *)com_port = '\r';
		}
	}
	s = "\n\r";
	while ((c = *s++) != 0) {
		while ((*((volatile unsigned char *)com_port +
				(UART_LSR << shift)) & UART_LSR_THRE) == 0)
		                ;
	        *(volatile unsigned char *)com_port = c;

		if (c == '\n') {
			while ((*((volatile unsigned char *)com_port +
				(UART_LSR << shift)) & UART_LSR_THRE) == 0)
					;
	        	*(volatile unsigned char *)com_port = '\r';
		}
	}
}
#endif	/* CONFIG_SERIAL_TEXT_DEBUG */

TODC_ALLOC();

void __init
platform_init(unsigned long r3, unsigned long r4, unsigned long r5,
	      unsigned long r6, unsigned long r7)
{
	bd_t *binfo;
	extern unsigned char __res[];

	parse_bootinfo(find_bootinfo());

	if ( r3 )
		memcpy( (void *)__res,(void *)(r3+KERNELBASE), sizeof(bd_t) );

	binfo = (bd_t *)__res;

#ifdef CONFIG_BLK_DEV_INITRD
	/* take care of initrd if we have one */
	if ( r4 )
	{
		initrd_start = r4 + KERNELBASE;
		initrd_end = r5 + KERNELBASE;
	}
#endif /* CONFIG_BLK_DEV_INITRD */

	/* take care of cmd line */
	if ( r6 )
	{
		*(char *)(r7+KERNELBASE) = 0;

		/* nothing but serial consoles... */
		if (!strstr((char *)(r6+KERNELBASE), "console="))
			sprintf(cmd_line, "%s console=ttyS0,%d",
				(char *)(r6+KERNELBASE),
			        binfo->bi_baudrate);
		else
			strcpy(cmd_line, (char *)(r6+KERNELBASE));
	}

	/* Map in board regs, etc. */
	psg_set_bat();

	isa_io_base = MPC10X_MAPB_ISA_IO_BASE;
	isa_mem_base = MPC10X_MAPB_ISA_MEM_BASE;
	pci_dram_offset = MPC10X_MAPB_DRAM_OFFSET;

	ppc_md.setup_arch = psg_setup_arch;
	ppc_md.show_cpuinfo = psg_show_cpuinfo;
	ppc_md.irq_cannonicalize = NULL;
	ppc_md.init_IRQ = psg_init_IRQ;
	ppc_md.get_irq = openpic_get_irq;
	ppc_md.init = NULL;

	ppc_md.restart = psg_restart;
	ppc_md.power_off = psg_power_off;
	ppc_md.halt = psg_halt;

	ppc_md.find_end_of_memory = psg_find_end_of_memory;
	ppc_md.setup_io_mappings = psg_map_io;

	ppc_md.calibrate_decr = psg_calibrate_decr;

	/* RTC is attached to RCS1... */
	TODC_INIT(TODC_TYPE_MK48T59, 0, 0, 0xff000000, 0);

	ppc_md.time_init = todc_time_init;
	ppc_md.set_rtc_time = todc_set_rtc_time;
	ppc_md.get_rtc_time = todc_get_rtc_time;

	ppc_md.nvram_read_val = todc_direct_read_val;
	ppc_md.nvram_write_val = todc_direct_write_val;

	ppc_md.heartbeat = NULL;
	ppc_md.heartbeat_reset = 0;
	ppc_md.heartbeat_count = 0;

#ifdef	CONFIG_SERIAL_TEXT_DEBUG
	ppc_md.progress = psg_progress;
#else	/* !CONFIG_SERIAL_TEXT_DEBUG */
	ppc_md.progress = NULL;
#endif	/* CONFIG_SERIAL_TEXT_DEBUG */

	return;
}
