
/*
 * sc_zeta.h 
 *
 * Board specific file for the "Zeta" reference platform 
 * designed by Switchcore.
 *
 * Based on fads.h, Copyright (c) 1998 Dan Malek (dmalek@jlc.net)
 * Written by Magnus Damm, Copyright (C) 2000 Switchcore AB. 
 * Modified by John W. Linville, Copyright (c) 2001 LVL7 Systems, Inc.
 */

#ifdef __KERNEL__

#ifndef __MACH_SC_ZETA_DEFS
#define __MACH_SZ_ZETA_DEFS

/*
 * 2000-nov-22: The SC_ZETA is using the following physical memorymap:
 *
 * ff040000 -> ff04ffff : CXE16 FIFO    connected to CS2
 * ff020000 -> ff02ffff : CXE16 REG     connected to CS1
 * ff010000 -> ff01ffff : PLD           connected to CS7
 * ff000000 -> ff00ffff : IMAP          internal in the cpu
 * fe000000 -> ffnnnnnn : flash         connected to CS0
 * f4000000 -> f400ffff : compact flash
 * 00000000 -> nnnnnnnn : sdram
 */

#define SC_ZETA_BOOT_FLASH_BASE 0xfe000000
#define SC_ZETA_BOOT_FLASH_SIZE 0x01000000 /* Max 16MByte */

#define SC_ZETA_PLD_REG_BASE    0xff010000 
#define SC_ZETA_PLD_REG_SIZE    0x00008000 /* 32KByte     */

#define SC_ZETA_CXE16_REG_BASE  0xff020000
#define SC_ZETA_CXE16_REG_SIZE  0x00010000 /* 64KByte     */

#define SC_ZETA_CXE16_FIFO_BASE 0xff040000
#define SC_ZETA_CXE16_FIFO_SIZE 0x00040000 /* 256KByte    */

#ifndef __ASSEMBLY__

/* A Board Information structure that is given to a program when
 * prom starts it up.
 */
#if defined(CONFIG_SC_ZETA_ELFBOOT)
/* This one is compatible w/ Elfboot... */
typedef struct bd_info {
        unsigned int    bi_memstart;    /* Memory start address */
        unsigned int    bi_memsize;     /* Memory (end) size in bytes */
        unsigned int    bi_intfreq;     /* Internal Freq, in Hz */
        unsigned int    bi_busfreq;     /* Bus Freq, in Hz */
        unsigned char   bi_enetaddr[6];
        unsigned int    bi_baudrate;
} bd_t;
#else
/* This one is compatible w/ PPCBoot... */
#include <asm/ppcboot.h>
#endif

/* Interrupt level assignments.
*/
#define FEC_INTERRUPT	SIU_LEVEL1	/* FEC interrupt */
#define PHY_INTERRUPT   SIU_IRQ2

#define IDE0_INTERRUPT  PCMCIA_INTERRUPT

/* override the default number of IDE hardware interfaces */
#define MAX_HWIFS       1

#define PCMCIA_MEM_ADDR 0xF4010000 
/*
 * Definitions for IDE0 Interface
 */
#define IDE0_BASE_OFFSET                0x0000  /* Offset in PCMCIA memory */
#define IDE0_DATA_REG_OFFSET            0x0000
#define IDE0_ERROR_REG_OFFSET           0x0081
#define IDE0_NSECTOR_REG_OFFSET         0x0082
#define IDE0_SECTOR_REG_OFFSET          0x0083
#define IDE0_LCYL_REG_OFFSET            0x0084
#define IDE0_HCYL_REG_OFFSET            0x0085
#define IDE0_SELECT_REG_OFFSET          0x0086
#define IDE0_STATUS_REG_OFFSET          0x0087
#define IDE0_CONTROL_REG_OFFSET         0x0106
#define IDE0_IRQ_REG_OFFSET             0x000A  /* not used */

/* We don't use the 8259.
*/
#define NR_8259_INTS	0

/* Get the immr
 */
extern __inline__ unsigned int get_immr_addr(void)
{
        unsigned int ret;
 
        asm volatile("mfspr %0,638" : "=r" (ret) :);
        ret &= 0xffff0000;
 
        return ret;
}
 
#if 0
#define IMAP_ADDR       (get_immr_addr())
#else
#define IMAP_ADDR       (0xff000000)
#endif
#define IMAP_SIZE       ((uint)(64 * 1024))

#define PCMCIA_MEM_SIZE         ((uint)(64 * 1024)) 

#endif /* __ASSEMBLY__ */
 
#endif

#endif

