/*
 * (C) Copyright 2002
 * John W. Linville, linville@lvl7.com
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <ppcboot.h>
#include <command.h>
#include <pci.h>
#include <i2c.h>
#include <asm/io.h>
#include <mpc824x.h>

#include "apollo.h"

#define PCI_READ_CONFIG_DWORD(bdf, reg, value) { \
	out_le32((unsigned int *)0xfec00cf8, \
	         (bdf | (reg & 0xfc) | 0x80000000)); \
	value = in_le32((unsigned int *)0xfee00cfc); \
}

#define PCI_WRITE_CONFIG_DWORD(bdf, reg, value) { \
	out_le32((unsigned int *)0xfec00cf8, \
	         (bdf | (reg & 0xfc) | 0x80000000)); \
	out_le32((unsigned int *)0xfee00cfc, value); \
}

#define PCI_READ_CONFIG_WORD(bdf, reg, value) { \
	out_le32((unsigned int *)0xfec00cf8, \
	         (bdf | (reg & 0xfc) | 0x80000000)); \
	value = in_le16((unsigned short *)(0xfee00cfc | (reg & 0x00000002))); \
}

#define PCI_WRITE_CONFIG_WORD(bdf, reg, value) { \
	out_le32((unsigned int *)0xfec00cf8, \
	         (bdf | (reg & 0xfc) | 0x80000000)); \
	out_le16((unsigned short *)(0xfee00cfc | (reg & 0x00000002)), value); \
}

#define PCI_READ_CONFIG_BYTE(bdf, reg, value) { \
	out_le32((unsigned int *)0xfec00cf8, \
	         (bdf | (reg & 0xfc) | 0x80000000)); \
	value = in_8((unsigned char *)(0xfee00cfc | (reg & 0x00000003))); \
}

#define PCI_WRITE_CONFIG_BYTE(bdf, reg, value) { \
	out_le32((unsigned int *)0xfec00cf8, \
	         (bdf | (reg & 0xfc) | 0x80000000)); \
	out_8((unsigned char *)(0xfee00cfc | (reg & 0x00000003)), value); \
}

/* Check board identity
 */
int checkboard (void)
{
	printf("Accton Apollo\n");

	return 1;
}


/*
 *NH
 *This is borrowed directly from sandpoint
 *It just figures out how much ram we have
 *and returns it to ppcboot.  Looking at
 *the zeta and aj platforms this appears to 
 *do real rm (upm) initalization, but
 *segment and bat registers seem to get initalized
 *from cpu_init.c and start.s respectively now
 *so I think I can do all my memory initalization
 *from my config file
 *NOTE: It appears this also re-configs the  
 *segment registers in pci space so that the 
 *segment boundry for dram ends at the real
 *end of memory rather than at the default
 *CFG_MAX_RAM_SIZE.  Bonus
 */ 
long initdram (int board_type)
{
   int              i, cnt;
   volatile uchar * base= CFG_SDRAM_BASE;
   volatile ulong * addr;
   ulong            save[32];
   ulong            val, ret  = 0;

   for (i=0, cnt=(CFG_MAX_RAM_SIZE / sizeof(long)) >> 1; cnt > 0; cnt >>= 1) {
                addr = (volatile ulong *)base + cnt;
                save[i++] = *addr;
                *addr = ~cnt;
   }

   addr = (volatile ulong *)base;
   save[i] = *addr;
   *addr = 0;

   if (*addr != 0) {
                *addr = save[i];
                goto Done;
   }

   for (cnt = 1; cnt <= CFG_MAX_RAM_SIZE / sizeof(long); cnt <<= 1) {
           addr = (volatile ulong *)base + cnt;
           val = *addr;
           *addr = save[--i];
           if (val != ~cnt) {
                   ulong new_bank0_end = cnt * sizeof(long) - 1;
                   ulong mear1  = mpc824x_mpc107_getreg(MEAR1);
                   ulong emear1 = mpc824x_mpc107_getreg(EMEAR1);
                   mear1 =  (mear1  & 0xFFFFFF00) |
                   ((new_bank0_end & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT);
                   emear1 = (emear1 & 0xFFFFFF00) |
                   ((new_bank0_end & MICR_ADDR_MASK) >> MICR_EADDR_SHIFT);
                   mpc824x_mpc107_setreg(MEAR1,  mear1);
                   mpc824x_mpc107_setreg(EMEAR1, emear1);

                   ret = cnt * sizeof(long);
                   goto Done;
           }
   }

   ret = CFG_MAX_RAM_SIZE;
Done:
   return ret;
}
