/*
 * (C) Copyright 2003
 * John W. Linville, linvjw@lvl7.com
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <ppcboot.h>
#include <command.h>
#include <pci.h>
#include <i2c.h>
#include <asm/io.h>
#include <mpc824x.h>

#include "bmw.h"

#define RTC_BASE                ( CFG_NVRAM_SIZE + CFG_NVRAM_BASE_ADDR )

#define RTC_SECONDS             ( RTC_BASE + 1 )

#define PCR                     0x800000E2

extern short pll_pci_to_mem_multiplier[];

extern void rtc_reset(void);
extern uchar rtc_read(unsigned int addr);

static __inline__ unsigned long get_dec(void)
{
	unsigned long val;

	asm volatile("mfdec %0" : "=r" (val) :);
	return val;
}

static __inline__ void set_dec(unsigned long val)
{
	asm volatile("mtdec %0" : : "r" (val));
}

/* Check board identity
 */
int checkboard (void)
{
	printf("Broadcom BCM95600PPC (BMW)\n");

	return 1;
}


/*
 *NH
 *This is borrowed directly from sandpoint
 *It just figures out how much ram we have
 *and returns it to ppcboot.  Looking at
 *the zeta and aj platforms this appears to 
 *do real rm (upm) initalization, but
 *segment and bat registers seem to get initalized
 *from cpu_init.c and start.s respectively now
 *so I think I can do all my memory initalization
 *from my config file
 *NOTE: It appears this also re-configs the  
 *segment registers in pci space so that the 
 *segment boundry for dram ends at the real
 *end of memory rather than at the default
 *CFG_MAX_RAM_SIZE.  Bonus
 */ 
long initdram (int board_type)
{
	int			i, cnt;
	volatile uchar * base= CFG_SDRAM_BASE;
	volatile ulong * addr;
	ulong		 save[32];
	ulong		 val, ret  = 0;

	for (i=0, cnt=(CFG_MAX_RAM_SIZE / sizeof(long)) >> 1;
	     cnt > 0; cnt >>= 1) {
		addr = (volatile ulong *)base + cnt;
		save[i++] = *addr;
		*addr = ~cnt;
	}

	addr = (volatile ulong *)base;
	save[i] = *addr;
	*addr = 0;

	if (*addr != 0) {
		*addr = save[i];
		goto Done;
	}

	for (cnt = 1; cnt <= CFG_MAX_RAM_SIZE / sizeof(long); cnt <<= 1) {
		addr = (volatile ulong *)base + cnt;
		val = *addr;
		*addr = save[--i];
		if (val != ~cnt) {
			ulong new_bank0_end = cnt * sizeof(long) - 1;
			ulong mear1  = mpc824x_mpc107_getreg(MEAR1);
			ulong emear1 = mpc824x_mpc107_getreg(EMEAR1);
			mear1 =  (mear1  & 0xFFFFFF00) |
			((new_bank0_end & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT);
			emear1 = (emear1 & 0xFFFFFF00) |
			((new_bank0_end & MICR_ADDR_MASK) >> MICR_EADDR_SHIFT);
			mpc824x_mpc107_setreg(MEAR1,  mear1);
			mpc824x_mpc107_setreg(EMEAR1, emear1);

			ret = cnt * sizeof(long);
			goto Done;
		}
	}

	ret = CFG_MAX_RAM_SIZE;
Done:
	return ret;
}

int board_pre_init(void)
{
	ulong val;
	short pci_mult;
	uchar pll_cfg;
	uchar start, cur;

	/* Configurations for RCS1 */
	CONFIG_READ_WORD(MCCR4, val);
	val |= 0x00020000;
	CONFIG_WRITE_WORD(MCCR4, val);

	/* Configurations for RCS2 and RCS3 */
	CONFIG_WRITE_WORD(0x800000d0, 0x84000000);
	CONFIG_WRITE_WORD(0x800000d4, 0x84000000);
	CONFIG_WRITE_WORD(0x800000d8, 0x0c000004);
	CONFIG_WRITE_WORD(0x800000dc, 0x00000001);

	/* Configurations for MIOCR1 and MIOCR2 */
	CONFIG_WRITE_BYTE(0x80000076, 0x04);
	CONFIG_WRITE_BYTE(0x80000077, 0x10);

	/* Configurations for PMCR2 */
	CONFIG_WRITE_BYTE(0x80000072, 0x20);

	/* Enable writes to NVRAM */
	*((char *)0x7c000003) = 0;

	/* Determine clock rate */
	rtc_reset();
	start = rtc_read(RTC_SECONDS);

	/* Catch start of next second... */
	while ((cur = rtc_read(RTC_SECONDS)) == start) ;
	start = cur;

	/* Set decrementor value... */
	set_dec(0xffffffff);

	/* Wait until end of this second... */
	while ((cur = rtc_read(RTC_SECONDS)) == start) ;

	/* Read decrementor value... */
	val = get_dec();
	val = (0xffffffff - val);
	val *= 4;

	/* Calculate clock multiplier... */
	CONFIG_READ_BYTE(PCR, pll_cfg);
	pll_cfg = (pll_cfg >> 3) & 0x1f;
	pci_mult = pll_pci_to_mem_multiplier[pll_cfg];

	/* Calculate frequency... (multiplier is x10) */
	CONFIG_SYS_CLK_FREQ = ((val / pci_mult) * 10);

	return (0);
}
