/*
 * (C) Copyright 2001
 * LVL7 Systems, Inc, 13000 Weston Pkwy, Suite 105, Cary, NC 27513
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <ppcboot.h>
#include <config.h>
#include "mpc8xx.h"
#include "aj.h"

/* ------------------------------------------------------------------------- */

int board_pre_init (void)
{
	volatile immap_t     *immap = (immap_t *)CFG_IMMR;
	volatile memctl8xx_t *memctl = &immap->im_memctl;
	volatile cpm8xx_t    *cpm = &immap->im_cpm;

#if 1 /*JWL*/
	/* Initialize chip selects CS1, CS2, CS3, CS4, CS5, CS6, CS7 */
	memctl->memc_br1 = 0;
	memctl->memc_or1 = 0;

	memctl->memc_br2 = 0;
	memctl->memc_or2 = 0;

	memctl->memc_br3 = 0;
	memctl->memc_or3 = 0;

	memctl->memc_br4 = 0;
	memctl->memc_or4 = 0;

	memctl->memc_br5 = 0;
	memctl->memc_or5 = 0;

	memctl->memc_br6 = 0;
	memctl->memc_or6 = 0;

	memctl->memc_br7 = 0;
	memctl->memc_or7 = 0;
#endif

	/* Map CPLD */
	memctl->memc_or2 = CPLD_OR2VALUE;
	memctl->memc_br2 = CPLD_BR2VALUE;

	/* Turn-on "OK" LED */
	*((volatile uint *)CPLD_LED_STATUS_REG) &= ~(1<<(CPLD_LED_OK_BIT));

	/* Pull-up CPU reset line */
	cpm->cp_pbdat |= 0x00040000;
	cpm->cp_pbdir |= 0x00040000;

#if 0 /* Not using DS1706S for watchdog functionality */
	/* Reset watchdog */
	cpm->cp_pbdat |= 0x00400000;
	cpm->cp_pbdir |= 0x00400000;
#endif

	return (0);
}

/* ------------------------------------------------------------------------- */

/*
 * Check Board Identity:
 *
 *    Probably should determine AJ24 vs. AJ48 here:
 *
 *       0 == AJ24
 *       1 == AJ48
 *
 *    For the time being, just return 0... -- JWL
 */

int checkboard (void)
{

  printf("LVL7 AmberJack\n");

  return 0;

}

/* ------------------------------------------------------------------------- */

/* sdram table by John W. Linville */

/* This table is derived from the AJ BSP for VxWorks(tm)
 */

const uint sdram_table[] =
{
	/* single read. (offset 0 in upm RAM) */
        0x1f07fc04, 0xeeaefc04, 0x11adfc04, 0xefbbbc00,
        0x1ff77c47, 0x1ff77c34, 0xefeabc34, 0x1fb57c35,

	/* burst read. (offset 8 in upm RAM) */
        0x1f07fc04, 0xeeaefc04, 0x10adfc04, 0xf0affc00,
        0xf0affc00, 0xf1affc00, 0xefbbbc00, 0x1ff77c47,
        0x1ff77c47, 0x03f0e407, 0xfff5cc47, 0xfffffc47,
        0xfffffc07, 0xfffffc07, 0xfffffc07, 0xfffffc07,

	/* single write. (offset 18 in upm RAM) */
        0x1f27fc04, 0xeeaebc00, 0x01b93c04, 0x1ff77c47,
        0x1ffffc07, 0xfffffc07, 0xfffffc07, 0xfffffc07,

	/* burst write. (offset 20 in upm RAM) */
        0x1f07fc04, 0xeeaebc00, 0x10ad7c00, 0xf0affc00,
        0xf0affc00, 0xe1bbbc04, 0x1ff77c47, 0x03acc047,
        0x0cacc007, 0x03bdc447, 0xfffffc07, 0xfffffc07,
        0xfffffc07, 0xfffffc07, 0xfffffc07, 0xfffffc07,

	/* refresh. (offset 30 in upm RAM) */
        0x1ff5fc84, 0xfffffc04, 0xfffffc04, 0xfffffc04,
        0xfffffc84, 0xfffffc07, 0xfffffc07, 0xfffffc07,
        0x7ffffc07, 0xfffffc07, 0xfffffc07, 0xfffffc07,

	/* exception. (offset 3c in upm RAM) */
        0x7ffffc07, 0xfffffc07, 0xfffffc07, 0xfffffc07 };

int _initsdram(uint base, uint noMbytes)
{
	volatile immap_t     *immap = (immap_t *)CFG_IMMR;
	volatile memctl8xx_t *memctl = &immap->im_memctl;
	volatile iop8xx_t    *iop = &immap->im_ioport;

	if ((noMbytes != 32) && (noMbytes != 64) && (noMbytes != 128))
	{
		return -1;
	}

#if 1 /*JWL*/
	/* Set memory frequency prescalar */
	memctl->memc_mptpr = SDRAM_MPTPRVALUE;

	/* Set initial MBMR value */
	memctl->memc_mbmr = SDRAM_MBMRVALUE0;

	/* Set SDRAM bank mux selectors */
	iop->iop_padir = SDRAM_PADIRVALUE;
	iop->iop_padat = SDRAM_PADATVALUE;

	/* Configure UPMB as SDRAM controller */
	upmconfig(UPMB, (uint *)sdram_table,sizeof(sdram_table)/sizeof(uint));

	/* Configure the refresh (mostly).  This needs to be
	 * based upon processor clock speed and optimized to provide
	 * the highest level of performance.  For multiple banks,
	 * this time has to be divided by the number of banks.
	 * Although it is not clear anywhere, it appears the
	 * refresh steps through the chip selects for this UPM
	 * on each refresh cycle.
	 * We have to be careful changing
	 * UPM registers after we ask it to run these commands.
	 *
	 * (Comment above preserved from FADS support -- JWL)
	 */

	memctl->memc_mar = SDRAM_MARVALUE;

	udelay(200);

	/* Now run the precharge/nop/mrs commands.
	*/

	memctl->memc_mcr = SDRAM_CS3_MCRVALUE0;

	udelay(200);

	/* Run 8 refresh cycles */

	memctl->memc_mcr = SDRAM_CS3_MCRVALUE1;

	udelay(200);

	memctl->memc_mbmr = SDRAM_MBMRVALUE1;
	memctl->memc_mcr = SDRAM_CS3_MCRVALUE2;

	udelay(200);

	memctl->memc_mbmr = SDRAM_MBMRVALUE0;

	udelay(200);

	memctl->memc_or3 = SDRAM_OR3VALUE;
	memctl->memc_br3 = SDRAM_BR3VALUE;

	udelay(200);

	if (noMbytes == 64)
	{
		return 0;
	}

	/* Now run the precharge/nop/mrs commands.
	*/

	memctl->memc_mcr = SDRAM_CS4_MCRVALUE0;

	udelay(200);

	/* Run 8 refresh cycles */

	memctl->memc_mcr = SDRAM_CS4_MCRVALUE1;

	udelay(200);

	memctl->memc_mbmr = SDRAM_MBMRVALUE1;
	memctl->memc_mcr = SDRAM_CS4_MCRVALUE2;

	udelay(200);

	memctl->memc_mbmr = SDRAM_MBMRVALUE0;

	udelay(200);

	memctl->memc_or4 = SDRAM_OR4VALUE;
	memctl->memc_br4 = SDRAM_BR4VALUE;

	udelay(200);
#endif

	return 0;
}

/* ------------------------------------------------------------------------- */

void _sdramdisable(void)
{
	volatile immap_t     *immap = (immap_t *)CFG_IMMR;
	volatile memctl8xx_t *memctl = &immap->im_memctl;

	memctl->memc_br3 = 0x00000000;
	memctl->memc_br4 = 0x00000000;

	/* maybe we should turn off upmb here or something */
}

/* ------------------------------------------------------------------------- */

int initsdram(uint base, uint *noMbytes)
{
#if 0
	uint m = 32;
#else
	uint m = 128;
#endif

	/* Need to read configuration via I2C bus... -- JWL */

	*noMbytes = m;

	if(!_initsdram(base, m))
	{

		return 0;
	}
	else
	{

		_sdramdisable();

		return -1;
	}
}

/* ------------------------------------------------------------------------- */

long int initdram (int board_type)
{
	uint	sdramsz;

	if (!initsdram(0x00000000, &sdramsz)) {

		printf("(%u MB SDRAM) ", sdramsz);

	} else {

		/********************************
		*DRAM ERROR, HALT PROCESSOR
		*********************************/
		while(1);

		return -1;
	}

	return (sdramsz << 20);
}

/* ------------------------------------------------------------------------- */

int testdram (void)
{
    /* TODO: XXX XXX XXX */
    printf ("test: 32 MB - ok (haha)\n");

    return (0);
}

/* ------------------------------------------------------------------------- */

const uint nP3400_table[] =
{
	/* single read. (offset 0 in upm RAM) */
        0xC0003004, 0x30003004, 0xF0003004, 0xF0003000,
        0xF0003005, 0x1FF77C34, 0xEFEABC34, 0x1FB57C35,

	/* burst read. (offset 8 in upm RAM) */
        0x1F07FC04, 0xEEAEFC04, 0x10ADFC04, 0xF0AFFC00,
        0xF0AFFC00, 0xF1AFFC00, 0xEFBBBC00, 0x1FF77C47,
        0x1FF77C47, 0x03F0E407, 0xFFF5CC47, 0xFFFFFC47,
        0xFFFFFC07, 0xFFFFFC07, 0xFFFFFC07, 0xFFFFFC07,

	/* single write. (offset 18 in upm RAM) */
        0xC0003C04, 0x30003C00, 0xF0003C05, 0x1FF77C47,
        0x1FFFFC07, 0xFFFFFC07, 0xFFFFFC07, 0xFFFFFC07,

	/* burst write. (offset 20 in upm RAM) */
        0x1F07FC04, 0xEEAEBC00, 0x10AD7C00, 0xF0AFFC00,
        0xF0AFFC00, 0xE1BBBC04, 0x1FF77C47, 0x03ACC047,
        0x0CACC007, 0x03BDC447, 0xFFFFFC07, 0xFFFFFC07,
        0xFFFFFC07, 0xFFFFFC07, 0xFFFFFC07, 0xFFFFFC07,

	/* refresh. (offset 30 in upm RAM) */
        0x1FF5FC84, 0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04,
        0xFFFFFC84, 0xFFFFFC07, 0xFFFFFC07, 0xFFFFFC07,
        0x7FFFFC07, 0xFFFFFC07, 0xFFFFFC07, 0xFFFFFC07,

	/* exception. (offset 3c in upm RAM) */
        0x7FFFFC07, 0xFFFFFC07, 0xFFFFFC07, 0xFFFFFC07 };

int misc_init_f (void)
{
	volatile immap_t     *immap = (immap_t *)CFG_IMMR;
	volatile memctl8xx_t *memctl = &immap->im_memctl;
#if 0 /*JWL*/
	volatile iop8xx_t    *iop = &immap->im_ioport;
#endif

	memctl->memc_mamr = CFG_MAMR;

	/* Configure UPMA as nP3400 controller */
	upmconfig(UPMA, (uint *)nP3400_table,sizeof(nP3400_table)/sizeof(uint));

#if 0 /*JWL*/
        /* bring SIL out of reset? */
	iop->iop_pcdat |= 0x00001000;
	iop->iop_pcdir |= 0x00001000;
#endif

	/* Map nP3400 */
	memctl->memc_or6 = NP3400_OR6VALUE;
	memctl->memc_br6 = NP3400_BR6VALUE;

	/* Map XSM */
	memctl->memc_or7 = XSM_OR7VALUE;
	memctl->memc_br7 = XSM_BR7VALUE;

	return (0);
}

void misc_init_r(bd_t *bd)
{
	volatile immap_t     *immap = (immap_t *)CFG_IMMR;
	ulong tim;

	/* set standard MPC8xx clock so kernel will not see 
         * a "random" time value
	 */

	/* convert to seconds value for 1/1/1970 00:00:01 */
	tim = mktime(1970, 1, 1, 0, 0, 1);

	immap->im_sitk.sitk_rtck = KAPWR_KEY;
	immap->im_sit.sit_rtc = tim;
}
