/*
 * (C) Copyright 2001
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <ppcboot.h>
#include <mpc8xx.h>

flash_info_t	flash_info[CFG_MAX_FLASH_BANKS]; /* info for FLASH chips	*/

#if defined(CFG_ENV_IS_IN_FLASH)
# ifndef  CFG_ENV_ADDR
#  define CFG_ENV_ADDR	(CFG_FLASH_BASE + CFG_ENV_OFFSET)
# endif
# ifndef  CFG_ENV_SIZE
#  define CFG_ENV_SIZE	CFG_ENV_SECT_SIZE
# endif
# ifndef  CFG_ENV_SECT_SIZE
#  define CFG_ENV_SECT_SIZE  CFG_ENV_SIZE
# endif
#endif

/*-----------------------------------------------------------------------
 * Functions
 */
static ulong flash_get_size (vu_long *addr, flash_info_t *info);

/*-----------------------------------------------------------------------
 */

unsigned long flash_init (void)
{


      /* comnt by kumar by default the flash is open */
	unsigned long size, offset = 0;
	int i;

	/* Init: enable write,
	 * or we cannot even write flash commands
	 */

	for (i=0; i<CFG_MAX_FLASH_BANKS; ++i) {
		flash_info[i].flash_id = FLASH_UNKNOWN;

		/* Re-do sizing to get full correct info */
		size = flash_get_size((vu_long *)(CFG_FLASH_BASE+offset),			                      &flash_info[i]);

		offset += size;
	}


#if 0

#if CFG_MONITOR_BASE >= CFG_FLASH_BASE
	/* monitor protection ON by default
	 *
	 * NOTE: This assumes that monitor is in _first_ part of _first_
	 *       flash bank.
	 */
	flash_protect(FLAG_PROTECT_SET,
		      CFG_MONITOR_BASE,
		      CFG_MONITOR_BASE+CFG_MONITOR_LEN-1,
		      &flash_info[0]);
#endif

#ifdef	CFG_ENV_IS_IN_FLASH
	/* ENV protection ON by default */
	flash_protect(FLAG_PROTECT_SET,
		      CFG_ENV_ADDR,
		      CFG_ENV_ADDR+CFG_ENV_SECT_SIZE-1,
		      &flash_info[0]);
#endif

#endif

	return (offset);
}

/*-----------------------------------------------------------------------
 */
void flash_print_info  (flash_info_t *info)
{
	int i;

	if (info->flash_id == FLASH_UNKNOWN) {
		printf ("missing or unknown FLASH type\n");
		return;
	}

	switch (info->flash_id & FLASH_VENDMASK) {
	case FLASH_MAN_INTEL:	printf ("Intel ");		break;
	default:		printf ("Unknown Vendor ");	break;
	}

	switch (info->flash_id & FLASH_TYPEMASK) {
	case FLASH_28F160S3:	printf ("28F160S3(4 MB)\n");
				break;
	case FLASH_28F160C3T:	printf ("28F160C3T(2 MB)\n");
				break;
	case FLASH_28F320C3B:	printf ("28F320C3B(4 MB)\n");
				break;
	case FLASH_28F320C3T:	printf ("28F320C3T(4 MB)\n");
				break;
	default:		printf ("Unknown Chip Type\n");
				break;
	}

	printf ("  Size: %ld MB in %d Sectors\n",
		info->size >> 20, info->sector_count);

	printf ("  Sector Start Addresses:");
	for (i=0; i<info->sector_count; ++i) {
		if ((i % 5) == 0)
			printf ("\n   ");
		printf (" %08lX%s",
			info->start[i],
			info->protect[i] ? " (RO)" : "     "
		);
	}
	printf ("\n");
}

/*-----------------------------------------------------------------------
 */


/*-----------------------------------------------------------------------
 */

/*
 * The following code cannot be run from FLASH!
 */

static ulong flash_get_size (vu_long *addr, flash_info_t *info)
{
	short i;
	ulong base = (ulong)addr;


	info->flash_id = FLASH_MAN_INTEL;
	info->flash_id += FLASH_28F160S3;
	info->sector_count = 32;
	info->size = 0x400000;

        for(i = 0; i < info->sector_count; i++)
		info->start[i] = base + (i * 0x00020000);
		
		

#if 0
	/* Write "Intelligent Identifier" command: read Manufacturer ID */
	*addr = 0x90909090;

	value = addr[0];
	switch (value & 0x00FF0000)
	{
	case (MT_MANUFACT & 0x00FF0000):
	case (INTEL_ALT_MANU & 0x00FF0000):	/* MT or => Intel */
		info->flash_id = FLASH_MAN_INTEL;
		break;
	default:
		info->flash_id = FLASH_UNKNOWN;
		info->sector_count = 0;
		info->size = 0;
		return (0);			/* no or unknown flash	*/
	}

	switch (value & 0x0000FFFF) {
	case (INTEL_ID_28F160C3B & 0x0000FFFF):
		info->flash_id += FLASH_28F160C3B;
		info->sector_count = 39;
		info->size = 0x00200000;
		bottom_boot_cnt = 8;
		break;				/* => 2 MB		*/

	case (INTEL_ID_28F160C3T & 0x0000FFFF):
		info->flash_id += FLASH_28F160C3T;
		info->sector_count = 39;
		info->size = 0x00200000;
		top_boot_cnt = 8;
		break;				/* => 2 MB		*/

	case (INTEL_ID_28F320C3B & 0x0000FFFF):
		info->flash_id += FLASH_28F320C3B;
		info->sector_count = 71;
		info->size = 0x00400000;
		bottom_boot_cnt = 8;
		break;				/* => 4 MB		*/

	case (INTEL_ID_28F320C3T & 0x0000FFFF):
		info->flash_id += FLASH_28F320C3T;
		info->sector_count = 71;
		info->size = 0x00400000;
		top_boot_cnt = 8;
		break;				/* => 4 MB		*/

	default:
		info->flash_id = FLASH_UNKNOWN;
		return (0);			/* => no or unknown flash */

	}

	/* set up sector start address table */
	/*Should be done through the region codes.*/

	for(i = 0; i < bottom_boot_cnt; i++) {
		//first 8 sectors are 8KB
		info->start[i] = base + (i * 0x00002000);
		info->protect[i] = 0;
	}
	base += (bottom_boot_cnt * 0x000002000);

	for (; i < (info->sector_count - top_boot_cnt); i++) {
		//Next 31 or 63 sectors
		info->start[i] = base + ((i - bottom_boot_cnt) * 0x00010000);
		/* don't know how to check sector protection */
		info->protect[i] = 0;
	}
	base += ((info->sector_count - bottom_boot_cnt - top_boot_cnt)
	        * 0x00010000);

	for(; i < info->sector_count; i++) {
		//last 8 sectors are 8KB
		ulong multiplier_fixup = info->sector_count - top_boot_cnt;
		info->start[i] = base + ((i - multiplier_fixup) * 0x00002000);
		info->protect[i] = 0;
	}

	/*
	 * Prevent writes to uninitialized FLASH.
	 */
	if (info->flash_id != FLASH_UNKNOWN) {
		addr = (vu_long *)info->start[0];

		*addr = 0xFFFFFFFF;	/* reset bank to read array mode */
	}

#endif
	return (info->size);
}

/*-----------------------------------------------------------------------
 */

#define FLASH_READ_ARRAY      		0x00FF00FF
#define FLASH_CLEAR_STATUS    		0x00500050
#define FLASH_PROGRAM	      		0x00400040	
#define FLASH_CLEAR_LOCK_GOOD_STATUS 	0x00800080
#define FLASH_BLOCK_ERASE		0x00200020
#define FLASH_BLOCK_ERASE_DATA		0x00D000D0

int write_buff(flash_info_t *info, uchar *src, 
		       ulong flash_address,
		       ulong block_length)
{

 unsigned int i;
 unsigned int new_value;
 unsigned int count;
 unsigned int source_address = (unsigned int)src;

 /* make sure that the block lenght is multiple of 4 since the flash */ 
 /* parts are arranged as 2*16-bits  				     */


 /* set the flash_address for testing */


 for(count = 0; count < 0x1000; count++);

 if(block_length % 4)
	block_length += (4 - (block_length % 4));

  /* clear the status register */

  *( unsigned int * )flash_address = FLASH_READ_ARRAY;
  for(count = 0; count < 0x10000; count++);

  *( unsigned int * )flash_address = FLASH_CLEAR_STATUS;
  for(count = 0; count < 0x10000; count++);

  for ( i = 0; i < block_length; i+=4)
  {
     
     /* read the value to program ... */
     
     new_value = *(unsigned int *) &((unsigned char *)source_address)[i];

     *(unsigned int *)( flash_address + i) = FLASH_PROGRAM;
     for(count = 0; count < 0x100; count++);

     *(unsigned int *)( flash_address + i) = new_value;
     for(count = 0; count < 0x100; count++);
 
     /* wait for completion */

     while((*(unsigned int *) (flash_address + i) & FLASH_CLEAR_LOCK_GOOD_STATUS) != FLASH_CLEAR_LOCK_GOOD_STATUS)
	{
		/* wait for completion from both devices */
	}
  }

  /* clear status reg, just in case */
  *(unsigned int *) flash_address = FLASH_CLEAR_STATUS;
  for(count = 0; count < 0x10000; count++);

  /* return flash to read array data */
  *(unsigned int *) flash_address = FLASH_READ_ARRAY;
  for(count = 0; count < 0x10000; count++);
  return 0;
} 


int flash_erase(flash_info_t *info, int s_first, int s_last)
{

    int count, sect;
    unsigned int start_address;

     /* set the start_address for testing */
     
    for(sect = s_first; sect <= s_last; ++sect)
    {
     	start_address = (unsigned int) (info->start[sect]);
     	for(count = 0; count < 0x1000; count++);

     	*(unsigned int *) start_address = FLASH_READ_ARRAY;
     	for(count = 0; count < 0x1000; count++);
	
     	*(unsigned int *) start_address = FLASH_CLEAR_STATUS;
     	for(count = 0; count < 0x1000; count++);

     	*(unsigned int *) start_address = FLASH_BLOCK_ERASE;
     	for(count = 0; count < 0x1000; count++);

     	*(unsigned int *) start_address = FLASH_BLOCK_ERASE_DATA;
     	for(count = 0; count < 0x1000; count++);

 
     	/* waite for complete */

     	while(((*(unsigned int *) start_address) & FLASH_CLEAR_LOCK_GOOD_STATUS) != FLASH_CLEAR_LOCK_GOOD_STATUS)
     		{

       		/* wait for completion from both devices */
     		}

     	if(*(unsigned int *) start_address & 0x00200020)
     		{
     		/* erase error */
     		/* clear status reg */
     
     		*(unsigned int *) start_address = FLASH_CLEAR_STATUS;
     		/* return flash to read array data */
     		*(unsigned int *) start_address = FLASH_READ_ARRAY;
     		return 1;

     		} 
     	/* clear status reg. */

     	*(unsigned int *) start_address = FLASH_CLEAR_STATUS;
     	for(count = 0; count < 0x10000; count++);

     	/* Return flash to read array data */
     	*(unsigned int *) start_address = FLASH_READ_ARRAY;
     	for(count = 0; count < 0x10000; count++);     
     }
     return 0;
}
