/*
 * (C) Copyright 2001,2002
 * LVL7 Systems, Inc, 13000 Weston Pkwy, Suite 105, Cary, NC 27513
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <ppcboot.h>
#include <config.h>
#include "mpc8xx.h"
#include "zeta.h"

#undef  SDRAM_CAS2
#undef  SDRAM_CAS3_LAZY_CS
#define SDRAM_CAS3_860P
#undef  SDRAM_CAS3_860P_LAZY_CS

/* ------------------------------------------------------------------------- */

int board_pre_init (void)
{
	volatile immap_t     *immap = (immap_t *)CFG_IMMR;
	volatile memctl8xx_t *memctl = &immap->im_memctl;
	volatile cpm8xx_t    *cpm = &immap->im_cpm;
        volatile pcmconf8xx_t *pcmctl = &immap->im_pcmcia;

	/* Initialize chip selects */

        memctl->memc_br1 = 0xFF020081;
        memctl->memc_or1 = 0xFFFF0400;

        memctl->memc_br2 = 0xFF040081;
        memctl->memc_or2 = 0xFFFC0400;

        memctl->memc_br3 = 0x00000000;
        memctl->memc_or3 = 0x00000000;

        memctl->memc_br4 = 0x00000000;
        memctl->memc_or4 = 0x00000000;

        memctl->memc_br5 = 0x00000000;
        memctl->memc_or5 = 0x00000000;

        memctl->memc_br6 = 0x00000000;
        memctl->memc_or6 = 0x00000000;

        memctl->memc_br7 = 0xFF010801;
        memctl->memc_or7 = 0xFFFF8750;

	/* pcmcia stuff */

        pcmctl->pcmc_pbr0 = 0xF4010000;
        pcmctl->pcmc_por0 = 0x78011541;

        pcmctl->pcmc_pbr1 = 0xF4010400;
        pcmctl->pcmc_por1 = 0x78011551;

        pcmctl->pcmc_pbr2 = 0xF4010800;
        pcmctl->pcmc_por2 = 0x78021559;

	/* Pull-up CPU reset line */
	cpm->cp_pbdat |= 0x00040000;
	cpm->cp_pbdir |= 0x00040000;

	return (0);
}

/* ------------------------------------------------------------------------- */

/*
 * Check Board Identity:
 *
 *    Probably should determine model here:
 *
 *       0 == SwitchCore CXE-RD16SX
 *       1 == SwitchCore CXE-RD1000TX
 *
 *    For the time being, just return 0... -- JWL
 */

int checkboard (void)
{

  printf("LVL7 SwitchCore\n");

  return 0;

}

/* ------------------------------------------------------------------------- */

/* sdram table by John W. Linville */

/* This table is derived from the AJ BSP for VxWorks(tm)
 */

const uint sdram_table[] = {

#ifdef SDRAM_CAS2
	/* single read. (offset 0 in upm RAM) */
        0xEF03FC24, 0x1F26FC24, 0xEEACFC04, 0x11B9BC04,
        0xEFF33C00, 0x1FF33C47, 0xFFFFFFFF, 0xFFFFFFFF,

	/* burst read. (offset 8 in upm RAM) */
        0x1F07FC24, 0xEEAEFC24, 0x10ADFC04, 0xF0AFFC00,
        0xF0AFFC00, 0xF1AFFC00, 0xEFBBBC00, 0x1FF77C47,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF,

	/* Precharge (Offset 0x16 in UPM RAM) */
        0xEFF33C04, 0x1FF33C45,

	/* single write. (offset 18 in upm RAM) */ 	
        0xEF03FC24, 0x1E26BC24, 0xE0AC3C00, 0x01B93C04,
        0x1FF33C47, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,

	/* burts write. (offset 20 in upm RAM) */
        0x1F07FC04, 0xEEAEBC00, 0x10AD7C00, 0xF0AFFC00,
        0xF0AFFC00, 0xE1BBBC04, 0x1FF77C47, 0xFFFFFFFF,

	/* NOP (Pos 0x2B) */
        0xEFFFFC84, 0x0FFFFC04, 0x1FFFFCC5, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,

	/* refresh. (offset 30 in upm RAM) */ 	
        0xEFFAFC84, 0x1FF5FC04, 0xFFFFFC04, 0xFFFFFC04,
        0xFFFFFC84, 0xFFFFFC87, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF,

	/* Mrs, Offset 0x3A */
        0xEFEABC34, 0x1FB57C35,

	/* exception. (offset 3c in upm RAM) */
        0xFFFFFC07, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
#endif

#ifdef SDRAM_CAS3_860P
	/* single read. (offset 0 in upm RAM) */
	0xef03fc24, 0x1f07fc24, 0xffaffc04, 0xefaefc04,
	0x1eadfc04, 0xe1bbbc04, 0x1ff77c00, 0xfffffc47,

	/* burst read. (offset 8 in upm RAM) */
	0xef03fc24, 0x1f07fc24, 0xfffffc04, 0xeffefc04,
	0x1efdfc04, 0xf0fffc04, 0xf0fffc00, 0xf0fffc00,
	0xf1fffc00, 0xfffffc00, 0xfffffc47, 0xffffffff,
	0xffffffff, 0xffffffff,

	/* Precharge (Offset 0x16 in UPM RAM) */
	0xeff33c04, 0x1ff33c45,

	/* single write. (offset 18 in upm RAM) */ 	
	0xef03fc24, 0x1f07fc24, 0xffaffc04, 0xeeaebc04,
	0x11ad7c04, 0xefbbbc04, 0x1ff77c00, 0xfffffc47,

	/* burts write. (offset 20 in upm RAM) */
	0xef03fc24, 0x1f07fc24, 0xfffffc04, 0xeefebc00,
	0x10fd7c00, 0xf0fffc00, 0xf0fffc00, 0xf1fffc04,

	/* NOP (Pos 0x2B) */
	0xfffffc04, 0xfffffc04, 0xfffffc47, 0xeffffc84,
	0x0ffffc04, 0x1ffffcc5, 0xffffffff, 0xffffffff,

	/* refresh. (offset 30 in upm RAM) */ 	
	0xfffffc04, 0xeffafc84, 0x1ff5fc04, 0xfffffc04,
	0xfffffc04, 0xfffffc04, 0xfffffc04, 0xfffffc04,
	0xfffffc84, 0xfffffc47,

	/* Mrs, Offset 0x3A */
	0xefeabc34, 0x1fb57c35,

	/* exception. (offset 3c in upm RAM) */
	0xfffffc04, 0xfffffc07, 0xffffffff, 0xffffffff
#endif

#ifdef SDRAM_CAS3_860P_LAZY_CS
	/* single read. (offset 0 in upm RAM) */
	0xEF03FC24, 0x0F27FC24, 0x0FAFFC04, 0x0FAEFC04,
	0x0EADFC04, 0x01BBBC04, 0x0FF77C00, 0x1FFFFC47,

	/* burst read. (offset 8 in upm RAM) */
	0xEF03FC24, 0x0F37FC24, 0x0FFFFC04, 0x0FFEFC04,
	0x0EFDFC04, 0x00FFFC04, 0x00FFFC00, 0x00FFFC00,
	0x01FFFC00, 0x0FFFFC00, 0x1FFFFC47, 0xFFFFFFFF,
	0xFFFFFFFF, 0xFFFFFFFF,

	/* Precharge (Offset 0x16 in UPM RAM) */
	0xEFF33C04, 0x1FF33C45,

	/* single write. (offset 18 in upm RAM) */ 	
	0xEF03FC24, 0x0F27FC24, 0x0FAFFC04, 0x0EAEBC04,
	0x01AD7C04, 0x0FBBBC04, 0x1FF77C00, 0xFFFFFC47,

	/* burts write. (offset 20 in upm RAM) */
	0xEF03FC24, 0x0F37FC24, 0x0FFFFC04, 0x0EFEBC00,
	0x00FD7C00, 0x00FFFC00, 0x00FFFC00, 0x01FFFC04,
	0x0FFFFC04, 0x0FFFFC04, 0x1FFFFC47,

	/* NOP (Pos 0x2B) */
	0xEFFFFC84,
	0x0FFFFC04, 0x1FFFFCC5, 0xFFFFFFFF, 0xFFFFFFFF,

	/* refresh. (offset 30 in upm RAM) */ 	
	0xFFFFFC04, 0xEFFAFC84, 0x1FF5FC04, 0xFFFFFC04,
	0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04,
	0xFFFFFC84, 0xFFFFFC47,

	/* Mrs, Offset 0x3A */
	0xEFEABC34, 0x1FB57C35,

	/* exception. (offset 3c in upm RAM) */
	0xFFFFFC04, 0xFFFFFC07, 0xFFFFFFFF, 0xFFFFFFFF
#endif

#ifdef SDRAM_CAS3_LAZY_CS
	/* single read. (offset 0 in upm RAM) */
	0xEF03FC24 /*0xFE03FC24*/, 0x0F03FC24, 0x0FAFFC04, 0x0FAFFC04,
	0x0FACFC04, 0x00AFFC04, 0x0FF33C00, 0x1FFFFC47,

	/* burst read. (offset 8 in upm RAM) */
	0xEF03FC24 /*0xFE03FC24*/, 0x0F03FC24, 0x0FFFFC04, 0x0FFFFC04,
	0x0FFCFC04, 0x00FFFC04, 0x00FFFC00, 0x00FFFC00,
	0x00FFFC00, 0x0FFFFC00 /*0x00FFFC00*/, 0x1FFFFC47, 0xFFFFFFFF,
	0xFFFFFFFF, 0xFFFFFFFF, 0xEFF33C04, 0x1FF33C45,

	/* single write. (offset 18 in upm RAM) */ 	
	0xEF03FC24, 0x0F03FC24, 0x0FAFFC04, 0x0FAFFC04,
	0x00AC3C04, 0x0FAFFC04, 0x0FF33C00, 0xFFFFFC47,

	/* burts write. (offset 20 in upm RAM) */
	0xEF03FC24, 0x0F03FC24, 0x0FFFFC04, 0x0FFFFC00 /*0x00FFFC00*/,
	0x00FC3C00, 0x00FFFC00, 0x00FFFC00, 0x00FFFC04,
	0x0FFFFC04, 0x0FFFFC04, 0x1FFFFC47, 0xEFFFFC84,
	0x0FFFFC04, 0x1FFFFCC5, 0xFFFFFFFF, 0xFFFFFFFF,

	/* refresh. (offset 30 in upm RAM) */ 	
	0xFFFFFC04, 0xFFFFFC84 /*0xEFFAFC84*/, 0x0FF0FC04, 0xFFFFFC04,
	0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04,
	0xFFFFFC84, 0xFFFFFC47, 0xEFEABC34, 0x1FB57C35,

	/* exception. (offset 3c in upm RAM) */
	0xFFFFFC04, 0xFFFFFC07, 0xFFFFFFFF, 0xFFFFFFFF
#endif

};

int _initsdram(uint base, uint noMbytes)
{
	volatile immap_t     *immap = (immap_t *)CFG_IMMR;
	volatile memctl8xx_t *memctl = &immap->im_memctl;
	volatile iop8xx_t    *iop = &immap->im_ioport;

	if ((noMbytes != 32) && (noMbytes != 64)
         && (noMbytes != 128) && (noMbytes != 256))
	{
		return -1;
	}

	/* Set SDRAM bank mux selectors */
	iop->iop_padir = SDRAM_PADIRVALUE;
	iop->iop_padat = SDRAM_PADATVALUE_ALARM; /* alarm LED */
	iop->iop_padat = SDRAM_PADATVALUE_FLASH;/* enable writing to FLASH */

	/* Configure UPMB as SDRAM controller */
	upmconfig(UPMB, (uint *)sdram_table,sizeof(sdram_table)/sizeof(uint));

	/* Configure the refresh (mostly).  This needs to be
	 * based upon processor clock speed and optimized to provide
	 * the highest level of performance.  For multiple banks,
	 * this time has to be divided by the number of banks.
	 * Although it is not clear anywhere, it appears the
	 * refresh steps through the chip selects for this UPM
	 * on each refresh cycle.
	 * We have to be careful changing
	 * UPM registers after we ask it to run these commands.
	 *
	 * (Comment above preserved from FADS support -- JWL)
	 */

	/* set memory frequeny prescalar*/
	memctl->memc_mptpr = SDRAM_MPTPRVALUE;

	/* set initial MBMR value */
	memctl->memc_mbmr = SDRAM_MBMRVALUE0;

	/* enable chip selects here */	
	memctl->memc_br5 = 0x000000C1;
	memctl->memc_or5 = 0xFE000A00;

	memctl->memc_br3 = 0x020000C1;
	memctl->memc_or3 = 0xFE000A00;

	udelay(200);

	/* After waiting, run NOP command 15 times from location 0x2B of UPMB */

	memctl->memc_mcr = 0x8080AF2B;
	memctl->memc_mcr = 0x80806F2B;

	udelay(200);

	/* run precharge command in location 0x16 of UPMB */
	memctl->memc_mcr = 0x8080A116;
	memctl->memc_mcr = 0x80806116;

	udelay(200);

	/* run refresh sequence with 8-beat refresh burst */
	memctl->memc_mcr = 0x8080A830;
	memctl->memc_mcr = 0x80806830;

	/* set MAR value for MRS command */
	memctl->memc_mar = SDRAM_MARVALUE;

	/* run MRS command in location 0x3A 0f UPMB */
	memctl->memc_mcr = 0x8080A13A;
	memctl->memc_mcr = 0x8080613A;

	udelay(200);

	/* After waiting, run NOP command 15 times from location 0x2B of UPMB */

	memctl->memc_mcr = 0x8080AF2B;
	memctl->memc_mcr = 0x80806F2B;

	memctl->memc_mbmr = SDRAM_MBMRVALUE1;

	return 0;
}

/* ------------------------------------------------------------------------- */

void _sdramdisable(void)
{
	volatile immap_t     *immap = (immap_t *)CFG_IMMR;
	volatile memctl8xx_t *memctl = &immap->im_memctl;

	memctl->memc_br3 = 0x00000000;
	memctl->memc_or3 = 0x00000000;

	memctl->memc_br5 = 0x00000000;
	memctl->memc_or5 = 0x00000000;

	/* maybe we should turn off upmb here or something */
}

/* ------------------------------------------------------------------------- */

int initsdram(uint base, uint *noMbytes)
{
	uint m = 64;

	/* Need to read configuration via I2C bus... -- JWL */

	*noMbytes = m;

	if(!_initsdram(base, m))
	{

		return 0;
	}
	else
	{

		_sdramdisable();

		return -1;
	}
}

/* ------------------------------------------------------------------------- */

long int initdram (int board_type)
{
	uint	sdramsz;

	if (!initsdram(0x00000000, &sdramsz)) {

		printf("(%u MB SDRAM) ", sdramsz);

	} else {

		/********************************
		*DRAM ERROR, HALT PROCESSOR
		*********************************/
		while(1);

		return -1;
	}

	return (sdramsz << 20);
}

/* ------------------------------------------------------------------------- */

int testdram (void)
{
    /* TODO: XXX XXX XXX */
    printf ("test: 32 MB - ok (haha)\n");

    return (0);
}

/* ------------------------------------------------------------------------- */

const uint cxe_table[] =
{

	/* single read. (offset 0 in upm RAM) */
	0xC0FFCC04, 0x00FFDC04, 0x00FFCC00, 0x30FFCC04,
	0xFFFFCC45, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,

	/* burst read, (offset 8 in upm RAM */
	0xC0FFCC04, 0x00FFDC04, 0x00FFCC00, 0x00FFDC04,
	0x00FFCC00, 0x00FFDC04, 0x00FFCC00, 0x00FFDC04,
	0x00FFCC00, 0x10FFCC45, 0xFFFFFFFF, 0xFFFFFFFF,
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,

	/* single write (offset 18 in upm RAM) */
	0xC0FFCC04, 0x00FFDC04, 0x00FFCC00, 0x30FFCC04,
	0xFFFFCC45, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,

	/* burst write (offset 20 in upm RAM) */
	0xC0FFCC04, 0x00FFDC04, 0x00FFCC00, 0x00FFDC04,
	0x00FFCC00, 0x00FFDC04, 0x00FFCC00, 0x00FFDC04,
	0x00FFCC00, 0x10FFCC45, 0xFFFFFFFF, 0xFFFFFFFF,
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,

	/* refresh. (offset 30 in upm RAM */
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,

	/* exception. (offset 3c in upm RAM) */
	0xFFFFFC07, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF

};

int misc_init_f (void)
{
	volatile immap_t     *immap = (immap_t *)CFG_IMMR;
	volatile memctl8xx_t *memctl = &immap->im_memctl;
	volatile cpm8xx_t    *cpm = &immap->im_cpm;

	memctl->memc_mamr = CFG_MAMR;

	/* Configure UPMA as CXE controller */
	upmconfig(UPMA, (uint *)cxe_table,sizeof(cxe_table)/sizeof(uint));

	return (0);
}

void misc_init_r(bd_t *bd)
{
	volatile immap_t     *immap = (immap_t *)CFG_IMMR;
	ulong tim;

	/* set standard MPC8xx clock so kernel will not see
	 * a "random" time value
	 */

	/* convert to seconds value for 1/1/1970 00:00:01 */
	tim = mktime(1970, 1, 1, 0, 0, 1);

	immap->im_sitk.sitk_rtck = KAPWR_KEY;
	immap->im_sit.sit_rtc = tim;
}
