/*
 *  Copyright (C) 1998	Dan Malek <dmalek@jlc.net>
 *  Copyright (C) 1999	Magnus Damm <kieraypc01.p.y.kie.era.ericsson.se>
 *  Copyright (C) 2000,2001,2002 Wolfgang Denk <wd@denx.de>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/* ppcboot - Startup Code for PowerPC based Embedded Boards
 *
 *
 * The processor starts at 0x00000100 and the code is executed
 * from flash. The code is organized to be at an other address
 * in memory, but as long we don't jump around before relocating.
 * board_init lies at a quite high address and when the cpu has
 * jumped there, everything is ok.
 * This works because the cpu gives the FLASH (CS0) the whole
 * address space at startup, and board_init lies as a echo of
 * the flash somewhere up there in the memorymap.
 *
 * board_init will change CS0 to be positioned at the correct
 * address and (s)dram will be positioned at address 0
 */
#include <config.h>
#include <mpc824x.h>
#include "version.h"

#define _LINUX_CONFIG_H 1	/* avoid reading Linux autoconf.h file	*/

#include <ppc_asm.tmpl>
#include <ppc_defs.h>

#include <asm/cache.h>
#include <asm/mmu.h>

#ifndef CONFIG_IDENT_STRING
#define CONFIG_IDENT_STRING ""
#endif

/* We don't want the MMU yet.
*/
#undef	MSR_KERNEL
/* FP, Machine Check and Recoverable Interr. */
#define MSR_KERNEL ( MSR_FP | MSR_ME | MSR_RI )

#ifdef CONFIG_MOUSSE

/* Macro for hiadjust and lo */
#define HIADJ(arg)      arg@ha
#define HI(arg)         arg@h
#define LO(arg)         arg@l

#undef LOADPTR  
#define LOADPTR(reg,const32) \
          addis reg,r0,HIADJ(const32); addi reg,reg,LO(const32)

#define PUTC(ch)        li r3, ch; bl SIO_OUT

#endif /* CONFIG_MOUSSE */

/*
 * Set up GOT: Global Offset Table
 *
 * Use r14 to access the GOT
 */
	START_GOT
	GOT_ENTRY(_GOT2_TABLE_)
	GOT_ENTRY(_FIXUP_TABLE_)

	GOT_ENTRY(_start)
	GOT_ENTRY(_start_of_vectors)
	GOT_ENTRY(_end_of_vectors)
	GOT_ENTRY(transfer_to_handler)

	GOT_ENTRY(_end)
	GOT_ENTRY(.bss)
#if defined(CONFIG_FADS)
	GOT_ENTRY(environment)
#endif
	END_GOT

/*
 * r3 - 1st arg to board_init(): IMMP pointer
 * r4 - 2nd arg to board_init(): boot flag
 */
	.text
	.long	0x27051956		/* PPCBOOT Magic Number			*/
	.globl	version_string
version_string:
	.ascii PPCBOOT_VERSION
	.ascii " (", __DATE__, " - ", __TIME__, ")"
	.ascii CONFIG_IDENT_STRING, "\0"

	. = EXC_OFF_SYS_RESET
	.globl	_start
_start:
	li	r21, BOOTFLAG_COLD	/* Normal Power-On: Boot from FLASH	*/
	b	boot_cold

	. = EXC_OFF_SYS_RESET + 0x10

	.globl	_start_warm
_start_warm:
	li	r21, BOOTFLAG_WARM	/* Software reboot			*/
	b	boot_warm

boot_cold:
boot_warm:

	/* Initialize machine status; enable machine check interrupt		*/
	/*----------------------------------------------------------------------*/
	li	r3, MSR_KERNEL		/* Set FP, ME, RI flags */
	mtmsr	r3
	mtspr	SRR1, r3		/* Make SRR1 match MSR */

	addis	r0,0,0x0000		/* lets make sure that r0 is really 0 */
	mtspr   HID0, r0		/* disable I and D caches */

	mfspr	r3, ICR			/* clear Interrupt Cause Register */

	mfmsr	r3			/* turn off address translation */
	addis	r4,0,0xffff
	ori	r4,r4,0xffcf
	and	r3,r3,r4
	mtmsr	r3
	isync
	sync				/* the MMU should be off... */

#ifdef CONFIG_MOUSSE
        bl      mousse_init
        bl      iommu_setup
#endif

in_flash:

	/*
	 * Setup BATs - cannot be done in C since we don't have a stack yet
	 */
	bl	setup_bats

	/* Enable MMU.
	 */
	mfmsr	r3
	ori	r3, r3, (MSR_IR | MSR_DR)
	mtmsr	r3

	/* Enable and invalidate data cache.
	 */
	mfspr	r3, HID0
	mr	r2, r3
	ori	r3, r3, HID0_DCE | HID0_DCI
	ori	r2, r2, HID0_DCE
	sync
	mtspr	HID0, r3
	mtspr	HID0, r2
	sync

	/* Allocate Initial RAM in data cache.
	 */
	lis	r3, CFG_INIT_RAM_ADDR@h
	ori	r3, r3, CFG_INIT_RAM_ADDR@l
	li	r2, 128
	mtctr	r2
1:
	dcbz	r0, r3
	addi	r3, r3, 32
	bdnz	1b

	/* Lock way0 in data cache.
	 */
	mfspr	r3, 1011
	lis	r2, 0xffff
	ori	r2, r2, 0xff1f
	and	r3, r3, r2
	ori	r3, r3, 0x0080
	sync
	mtspr	1011, r3

	/*
	 * Thisk the stack pointer *somewhere* sensible. Doesnt
	 * matter much where as we'll move it when we relocate
	 */
	lis	r1, (CFG_INIT_RAM_ADDR + CFG_INIT_DATA_OFFSET)@h
	ori	r1, r1, (CFG_INIT_RAM_ADDR + CFG_INIT_DATA_OFFSET)@l

	li	r0, 0			/* Make room for stack frame header and	*/
	stwu	r0, -4(r1)		/* clear final stack frame so that	*/
	stwu	r0, -4(r1)		/* stack backtraces terminate cleanly	*/

	/* let the C-code set up the rest					*/
	/*									*/
	/* Be careful to keep code relocatable !				*/
	/*----------------------------------------------------------------------*/

	GET_GOT			/* initialize GOT access			*/

	/* r3: IMMR */
	bl	cpu_init_f	/* run low-level CPU init code     (from Flash)	*/

	mr	r3, r21
	/* r3: BOOTFLAG */
	bl	board_init_f	/* run 1st part of board init code (from Flash) */



	.globl	_start_of_vectors
_start_of_vectors:

/* Machine check */
	STD_EXCEPTION(EXC_OFF_MACH_CHCK, MachineCheck, MachineCheckException)

/* Data Storage exception.  "Never" generated on the 860. */
	STD_EXCEPTION(EXC_OFF_DATA_STOR, DataStorage, UnknownException)

/* Instruction Storage exception.  "Never" generated on the 860. */
	STD_EXCEPTION(EXC_OFF_INS_STOR, InstStorage, UnknownException)

/* External Interrupt exception. */
	STD_EXCEPTION(EXC_OFF_EXTERNAL, ExtInterrupt, external_interrupt)

/* Alignment exception. */
	. = EXC_OFF_ALIGN
Alignment:
	EXCEPTION_PROLOG
	mfspr	r4,DAR
	stw	r4,_DAR(r21)
	mfspr	r5,DSISR
	stw	r5,_DSISR(r21)
	addi	r3,r1,STACK_FRAME_OVERHEAD
	li	r20,MSR_KERNEL
	rlwimi	r20,r23,0,16,16		/* copy EE bit from saved MSR */
	lwz	r6,GOT(transfer_to_handler)
	mtlr	r6
	blrl
.L_Alignment:
	.long	AlignmentException - _start + EXC_OFF_SYS_RESET
	.long	int_return - _start + EXC_OFF_SYS_RESET

/* Program check exception */
	. = EXC_OFF_PROGRAM
ProgramCheck:
	EXCEPTION_PROLOG
	addi	r3,r1,STACK_FRAME_OVERHEAD
	li	r20,MSR_KERNEL
	rlwimi	r20,r23,0,16,16		/* copy EE bit from saved MSR */
	lwz	r6,GOT(transfer_to_handler)
	mtlr	r6
	blrl
.L_ProgramCheck:
	.long	ProgramCheckException - _start + EXC_OFF_SYS_RESET
	.long	int_return - _start + EXC_OFF_SYS_RESET

	/* No FPU on MPC8xx. This exception is not supposed to happen.
	*/
	STD_EXCEPTION(EXC_OFF_FPUNAVAIL, FPUnavailable, UnknownException)

	/* I guess we could implement decrementer, and may have
	 * to someday for timekeeping.
	 */
	STD_EXCEPTION(EXC_OFF_DECR, Decrementer, timer_interrupt)
	STD_EXCEPTION(0xa00, Trap_0a, UnknownException)
	STD_EXCEPTION(0xb00, Trap_0b, UnknownException)

	. = 0xc00
/*
 * r0 - SYSCALL number
 * r3-... arguments
 */
SystemCall:
	addis	r11,r0,0		/* get functions table addr */
	ori	r11,r11,0		/* Note: this code is patched in trap_init */
	addis	r12,r0,0		/* get number of functions */
	ori	r12,r12,0

	cmplw	0, r0, r12
	bge	1f

	rlwinm	r0,r0,2,0,31		/* fn_addr = fn_tbl[r0] */
	add	r11,r11,r0
	lwz	r11,0(r11)

	li	r12,0xd00-4*3		/* save LR & SRRx */
	mflr	r0
	stw	r0,0(r12)
	mfspr	r0,SRR0
	stw	r0,4(r12)
	mfspr	r0,SRR1
	stw	r0,8(r12)

	li	r12,0xc00+_back-SystemCall
	mtlr	r12
	mtspr	SRR0,r11

1:	SYNC
	rfi

_back:

	mfmsr	r11			/* Disable interrupts */
	li	r12,0
	ori	r12,r12,MSR_EE
	andc	r11,r11,r12
	SYNC				/* Some chip revs need this... */
	mtmsr	r11
	SYNC

	li	r12,0xd00-4*3		/* restore regs */
	lwz	r11,0(r12)
	mtlr	r11
	lwz	r11,4(r12)
	mtspr	SRR0,r11
	lwz	r11,8(r12)
	mtspr	SRR1,r11

	SYNC
	rfi

	STD_EXCEPTION(EXC_OFF_TRACE, SingleStep, UnknownException)

	STD_EXCEPTION(EXC_OFF_FPUNASSIST, Trap_0e, UnknownException)
	STD_EXCEPTION(EXC_OFF_PMI, Trap_0f, UnknownException)

	STD_EXCEPTION(EXC_OFF_ITME, InstructionTransMiss, UnknownException)
	STD_EXCEPTION(EXC_OFF_DLTME, DataLoadTransMiss, UnknownException)
	STD_EXCEPTION(EXC_OFF_DSTME, DataStoreTransMiss, UnknownException)
	STD_EXCEPTION(EXC_OFF_IABE, InstructionBreakpoint, UnknownException)
	STD_EXCEPTION(EXC_OFF_SMIE, SysManageInt, UnknownException)
	STD_EXCEPTION(0x1500, Reserved5, UnknownException)
	STD_EXCEPTION(0x1600, Reserved6, UnknownException)
	STD_EXCEPTION(0x1700, Reserved7, UnknownException)
	STD_EXCEPTION(0x1800, Reserved8, UnknownException)
	STD_EXCEPTION(0x1900, Reserved9, UnknownException)
	STD_EXCEPTION(0x1a00, ReservedA, UnknownException)
	STD_EXCEPTION(0x1b00, ReservedB, UnknownException)
	STD_EXCEPTION(0x1c00, ReservedC, UnknownException)
	STD_EXCEPTION(0x1d00, ReservedD, UnknownException)
	STD_EXCEPTION(0x1e00, ReservedE, UnknownException)
	STD_EXCEPTION(0x1f00, ReservedF, UnknownException)

	STD_EXCEPTION(EXC_OFF_RMTE, RunModeTrace, UnknownException)

	.globl	_end_of_vectors
_end_of_vectors:


	. = 0x3000
#ifdef CONFIG_MOUSSE
#define USER_I_CACHE_ENABLE 1
        /* Initialize the CPCI Mousse card */
        .global mousse_init
mousse_init:
        /* Zero-out registers */
        addis   r0,r0,0
        mtspr   SPRG0,r0
        mtspr   SPRG1,r0
        mtspr   SPRG2,r0
        mtspr   SPRG3,r0

        /* Set MPU/MSR to a known state. Turn on FP */
        LOADPTR (r3, MSR_FP)
        sync
        mtmsr   r3
        isync
        /* Init the floating point control/status register */
        mtfsfi  7,0x0
        mtfsfi  6,0x0
        mtfsfi  5,0x0
        mtfsfi  4,0x0
        mtfsfi  3,0x0
        mtfsfi  2,0x0
        mtfsfi  1,0x0
        mtfsfi  0,0x0
        isync

        /* Set MPU/MSR to a known state. Turn off FP */

#if 0   /* Code removed so FP stays on */
        andi.   r3, r3, 0
        sync
        mtmsr   r3
        isync
#endif

        /* Init the Segment registers */
        andi.   r3, r3, 0
        isync
        mtsr    0,r3
        isync
        mtsr    1,r3
        isync
        mtsr    2,r3
        isync
        mtsr    3,r3
        isync
        mtsr    4,r3
        isync
        mtsr    5,r3
        isync
        mtsr    6,r3
        isync
        mtsr    7,r3
        isync
        mtsr    8,r3
        isync
        mtsr    9,r3
        isync
        mtsr    10,r3
        isync
        mtsr    11,r3
        isync
        mtsr    12,r3
        isync
        mtsr    13,r3
        isync
        mtsr    14,r3
        isync
        mtsr    15,r3
        isync

        /* Turn off data and instruction cache control bits */
       
        mfspr   r3, HID0
        isync
        rlwinm  r4, r3, 0, 18, 15       /* r4 has ICE and DCE bits cleared */
        sync
        isync
        mtspr   HID0, r4                /* HID0 = r4 */
        isync

        /* Get cpu type */
        mfspr   r28, PVR
        rlwinm  r28, r28, 16, 16, 31

        /* invalidate the MPU's data/instruction caches */
        lis     r3, 0x0
        cmpli   0, 0, r28, CPU_TYPE_603
        beq     cpuIs603
        cmpli   0, 0, r28, CPU_TYPE_603E
        beq     cpuIs603
        cmpli   0, 0, r28, CPU_TYPE_603P
        beq     cpuIs603
        cmpli   0, 0, r28, CPU_TYPE_604R
        bne     cpuNot604R

cpuIs604R:
        lis     r3, 0x0
        mtspr   HID0, r3                /* disable the caches */
        isync
        ori     r4, r4, 0x0002          /* disable BTAC by setting bit 30 */

cpuNot604R:
        ori     r3, r3, (HID0_ICFI | HID0_DCI)

cpuIs603:
        ori     r3, r3, (HID0_ICE | HID0_DCE)
        or      r4, r4, r3              /* set bits */
        sync
        isync
        mtspr   HID0, r4                /* HID0 = r4 */
        andc    r4, r4, r3              /* clear bits */
        isync
        cmpli   0, 0, r28, CPU_TYPE_604
        beq     cpuIs604
        cmpli   0, 0, r28, CPU_TYPE_604E
        beq     cpuIs604
        cmpli   0, 0, r28, CPU_TYPE_604R
        beq     cpuIs604
        mtspr   HID0, r4
        isync

#ifdef USER_I_CACHE_ENABLE
        b       instCacheOn603
#else
        b       cacheEnableDone
#endif

cpuIs604:
        LOADPTR (r5, 0x1000)            /* loop count, 0x1000 */
        mtspr   CTR, r5
loopDelay:
        nop
        bdnz    loopDelay
        isync
        mtspr   HID0, r4
        isync

        /* turn the Instruction cache ON for faster FLASH ROM boots */

#ifdef USER_I_CACHE_ENABLE

        ori     r4, r4, (HID0_ICE | HID0_ICFI)
        isync                           /* Synchronize for ICE enable */
        b       writeReg4
instCacheOn603:
        ori     r4, r4, (HID0_ICE | HID0_ICFI)
        rlwinm  r3, r4, 0, 21, 19       /* clear the ICFI bit */

        /*
         * The setting of the instruction cache enable (ICE) bit must be
         * preceded by an isync instruction to prevent the cache from being
         * enabled or disabled while an instruction access is in progress.
         */
        isync
writeReg4:
        mtspr   HID0, r4                /* Enable Instr Cache & Inval cache */
        cmpli   0, 0, r28, CPU_TYPE_604
        beq     cacheEnableDone
        cmpli   0, 0, r28, CPU_TYPE_604E
        beq     cacheEnableDone

        mtspr   HID0, r3                /* using 2 consec instructions */
                                        /* PPC603 recommendation */
#endif


cacheEnableDone:
        /* Detect map A or B */
        addis   r5,r0, HI(CHRP_REG_ADDR)
        addis   r6,r0, HI(CHRP_REG_DATA)
        LOADPTR (r7, KAHLUA_ID)         /* Kahlua PCI controller ID */
        LOADPTR (r8, BMC_BASE)

        stwbrx  r8,0,(r5)
        lwbrx   r3,0,(r6)               /* Store read value to r3 */
        cmp     0,0,r3,r7
        beq     cr0, X4_KAHLUA_START
        LOADPTR (r5, PREP_REG_ADDR)
        LOADPTR (r6, PREP_REG_DATA)

X4_KAHLUA_START:

// Errata to address latency timer RP 7/20/99

        LOADPTR(r3,BMC_BASE)            // Select LATENCY_TIMER
        ori     r3,r3,0x000d
        li      r4, 0x20                // Set to 0x20

        stwbrx  r3,0,r5                 //Set REG_ADDR
        sync
        stb     r4, 1(r6)
        sync

        LOADPTR(r3,BMC_BASE)            // Select CACHE_LINE_SIZE
        ori     r3,r3,0x000c
        li      r4, 0x08                // Set to 0x08

        stwbrx  r3,0,r5
        sync
        stb     r4, 0(r6)
        sync

        LOADPTR(r3,BMC_BASE)            // Select PCI_CMD
        ori     r3,r3,0x0004
        li      r4, 0x0006              // Set to 06 (Memory Space)
        stwbrx  r3,0,r5
        sync
        sthbrx  r4, 0, r6
        sync

        LOADPTR(r3, BMC_BASE)           // Select PCI_STAT
        ori     r3,r3,0x0006
        stwbrx  r3,0,r5
        sync

        li      r3, 0x0002
        lhbrx   r4, r3, r6              // Get old PCI_STAT
        sync
        ori     r4, r4, 0xffff     // Writing all ones will clear all bits in
        sthbrx  r4, r3, r6         // write the modified data to CONFIG_DATA
#if 0

        LOADPTR (r3, MPC107_PCI_CMD)    /* PCI command reg */
        stwbrx  r3,0,r5
        li      r4, 6                   /* Command register value */
        sthbrx  r4, 0, r6

        LOADPTR (r3, MPC107_PCI_STAT)   /* PCI status reg */
        stwbrx  r3,0,r5
        li      r4, -1                  /* Write-to-clear all bits */
        li      r3, 2                   /* PCI_STATUS is at +2 offset */
        sthbrx  r4, r3, r6
#endif

        /*-------PROC_INT1_ADR */

        LOADPTR (r3, PROC_INT1_ADR)     /* Processor I/F Config 1 reg. */
        stwbrx  r3,0,r5
        /*      LOADPTR (r4, 0xff141a98) */

        lis     r4, 0x00000004          // processor type = 603/750

        ori     r4, r4, 0x1000          // enable Flash write
        ori     r4, r4, 0x0800          // enable MCP* assertion
        ori     r4, r4, 0x0200          // enable data bus parking
        ori     r4, r4, 0x0040          // enable PCI store gathering
        ori     r4, r4, 0x0010          // RESERVED -- Set
        ori     r4, r4, 0x0008          // enable address bus parking

        stwbrx  r4,0,r6

        /*-------PROC_INT2_ADR */
        LOADPTR (r3, PROC_INT2_ADR)     /* Processor I/F Config 2 reg. */
        stwbrx  r3,0,r5

        lis     r4, 0x0000
//      ori    r4, r4, 0x2000           // Disable Serialize Config cycles
//      ori    r4, r4, 0x0800           // Disable PCI Snoop cycles
        ori    r4, r4, 0x0400           // FF0 is Local ROM
//      ori    r4, r4, 0x0200           // Flash write lockout
        ori    r4, r4, 0x0000           // snoop wt states = 0
//      ori    r4, r4, 0x0004           // snoop wt states = 1
//      ori    r4, r4, 0x0008           // snoop wt states = 2
//      ori    r4, r4, 0x000c           // snoop wt states = 3
        ori     r4, r4, 0x0000          // addr. phase wt states = 0
//      ori     r4, r4, 0x0004          // addr. phase wt states = 1
//      ori     r4, r4, 0x0008          // addr. phase wt states = 2
//      ori     r4, r4, 0x000c          // addr. phase wt states = 3
        stwbrx  r4,0,r6


        /* Reset default OK */

        /*-------MCCR1 */
#ifdef INCLUDE_ECC
#define MC_ECC                          1
#else /* INCLUDE_ECC */
#define MC_ECC                          0
#endif /* INCLUDE_ECC */

#define MC1_ROMNAL                      8               /* 0-15 */
#define MC1_ROMFAL                      8               /* 0-31 */
#define MC1_DBUS_SIZE                   0               /* 0-3 */
#define MC1_BURST                       0               /* 0-1 */
#define MC1_MEMGO                       0               /* 0-1 */
#define MC1_SREN                        1               /* 0-1 */
#define MC1_RAM_TYPE                    0               /* 0-1 */
#define MC1_PCKEN                       MC_ECC          /* 0-1 */
#define MC1_BANKBITS                    0x5555          /* 2 bits/bank 7-0 */
        LOADPTR (r3, MEM_CONT1_ADR)     /* Set MCCR1 (F0) */
        stwbrx  r3,0,r5
        LOADPTR(r4, \
                MC1_ROMNAL << 28 | MC1_ROMFAL << 23 | \
                MC1_DBUS_SIZE << 21 | MC1_BURST << 20 | \
                MC1_MEMGO << 19 | MC1_SREN << 18 | \
                MC1_RAM_TYPE << 17 | MC1_PCKEN << 16 | \
                MC1_BANKBITS)
        stwbrx  r4, 0, r6

        /*------- MCCR2 */

#define MC2_TS_WAIT_TIMER               0               /* 0-7 */
#define MC2_ASRISE                      8               /* 0-15 */
#define MC2_ASFALL                      4               /* 0-15 */
#define MC2_INLINE_PAR_NOT_ECC          0               /* 0-1 */
#define MC2_WRITE_PARITY_CHK_EN         MC_ECC          /* 0-1 */
#define MC2_INLRD_PARECC_CHK_EN         MC_ECC          /* 0-1 */
#define MC2_ECC_EN                      0               /* 0-1 */
#define MC2_EDO                         0               /* 0-1 */
#define MC2_REFINT                      0xf5            /* 0-0x3fff */
#define MC2_RSV_PG                      0               /* 0-1 */
#define MC2_RMW_PAR                     MC_ECC          /* 0-1 */
        LOADPTR (r3, MEM_CONT2_ADR)     /* Set MCCR2 (F4) */
        stwbrx  r3,0,r5
        LOADPTR(r4, \
                MC2_TS_WAIT_TIMER << 29 | MC2_ASRISE << 25 | \
                MC2_ASFALL << 21 | MC2_INLINE_PAR_NOT_ECC << 20 | \
                MC2_WRITE_PARITY_CHK_EN << 19 | \
                MC2_INLRD_PARECC_CHK_EN << 18 | \
                MC2_ECC_EN << 17 | MC2_EDO << 16 | \
                MC2_REFINT << 2 | MC2_RSV_PG << 1 | MC2_RMW_PAR)
        stwbrx  r4,0,r6

        /*------- MCCR3 */
#define MC_BSTOPRE                      0x079           /* 0-0x7ff */

#define MC3_BSTOPRE_U                   (MC_BSTOPRE >> 4 & 0xf)
#define MC3_REFREC                      8               /* 0-15 */
#define MC3_RDLAT                       (4+MC_ECC)      /* 0-15 */
#define MC3_CPX                         0               /* 0-1 */
#define MC3_RAS6P                       0               /* 0-15 */
#define MC3_CAS5                        0               /* 0-7 */
#define MC3_CP4                         0               /* 0-7 */
#define MC3_CAS3                        0               /* 0-7 */
#define MC3_RCD2                        0               /* 0-7 */
#define MC3_RP1                         0               /* 0-7 */
        LOADPTR (r3, MEM_CONT3_ADR)     /* Set MCCR3 (F8) */
        stwbrx  r3,0,r5
        LOADPTR(r4, \
                MC3_BSTOPRE_U << 28 | MC3_REFREC << 24 | \
                MC3_RDLAT << 20 | MC3_CPX << 19 | \
                MC3_RAS6P << 15 | MC3_CAS5 << 12 | MC3_CP4 << 9 | \
                MC3_CAS3 << 6 | MC3_RCD2 << 3 | MC3_RP1)
        stwbrx  r4,0,r6

        /*------- MCCR4 */

#define MC4_PRETOACT                    3               /* 0-15 */
#define MC4_ACTOPRE                     5               /* 0-15 */
#define MC4_WMODE                       0               /* 0-1 */
#define MC4_INLINE                      MC_ECC          /* 0-1 */
#define MC4_REGISTERED                  (1-MC_ECC)      /* 0-1 */
#define MC4_BSTOPRE_UU                  (MC_BSTOPRE >> 8 & 3)
#define MC4_REGDIMM                     0               /* 0-1 */
#define MC4_SDMODE_CAS                  3               /* 0-7 */
#define MC4_SDMODE_WRAP                 0               /* 0-1 */
#define MC4_SDMODE_BURST                2               /* 0-7 */
#define MC4_ACTORW                      3               /* 0-15 */
#define MC4_BSTOPRE_L                   (MC_BSTOPRE & 0xf)
        LOADPTR (r3, MEM_CONT4_ADR)     /* Set MCCR4 (FC) */
        stwbrx  r3,0,r5
        LOADPTR(r4, \
                MC4_PRETOACT << 28 | MC4_ACTOPRE << 24 | \
                MC4_WMODE << 23 | MC4_INLINE << 22 | \
                MC4_REGISTERED << 20 | MC4_BSTOPRE_UU << 18 | \
                MC4_REGDIMM << 15 | MC4_SDMODE_CAS << 12 | \
                MC4_SDMODE_WRAP << 11 | MC4_SDMODE_BURST << 8 | \
                MC4_ACTORW << 4 | MC4_BSTOPRE_L)
        stwbrx  r4,0,r6

#ifdef INCLUDE_ECC
        /*------- MEM_ERREN1 */
        LOADPTR (r3, MEM_ERREN1_ADR)    /* Set MEM_ERREN1 (c0) */
        stwbrx  r3,0,r5
        lwbrx   r4,0,r6
        ori     r4,r4,4                 /* Set MEM_PERR_EN */
        stwbrx  r4,0,r6
#endif /* INCLUDE_ECC */
        /*------- MSAR/MEAR */
        LOADPTR (r3, MEM_START1_ADR)    /* Set MSAR1 (80) */
        stwbrx  r3,0,r5
        LOADPTR (r4, 0xc0804000)
        stwbrx  r4,0,r6

        LOADPTR (r3, MEM_START2_ADR)    /* Set MSAR2 (84) */
        stwbrx  r3,0,r5
        LOADPTR (r4, 0xc0804000)
        stwbrx  r4,0,r6

        LOADPTR (r3, XMEM_START1_ADR)   /* Set MESAR1 (88) */
        stwbrx  r3,0,r5
        LOADPTR (r4, 0x00000000)
        stwbrx  r4,0,r6

        LOADPTR (r3, XMEM_START2_ADR)   /* Set MESAR2 (8c) */
        stwbrx  r3,0,r5
        LOADPTR (r4, 0x01010101)
        stwbrx  r4,0,r6

        LOADPTR (r3, MEM_END1_ADR)      /* Set MEAR1 (90) */
        stwbrx  r3,0,r5
        LOADPTR (r4, 0xffbf7f3f)
        stwbrx  r4,0,r6

        LOADPTR (r3, MEM_END2_ADR)      /* Set MEAR2 (94) */
        stwbrx  r3,0,r5
        LOADPTR (r4, 0xffbf7f3f)
        stwbrx  r4,0,r6

        LOADPTR (r3, XMEM_END1_ADR)     /* MEEAR1 (98) */
        stwbrx  r3,0,r5
        LOADPTR (r4, 0x00000000)
        stwbrx  r4,0,r6

        LOADPTR (r3, XMEM_END2_ADR)     /* MEEAR2 (9c) */
        stwbrx  r3,0,r5
        LOADPTR (r4, 0x01010101)
        stwbrx  r4,0,r6

        /*-------ODCR */
        LOADPTR (r3, OUT_DRV_CONT)      /* Set ODCR */
        stwbrx  r3,0,r5
        li      r4, 0x7f
#if 0
li      r4, 0
        ori     r4, r4, 0x80            // PCI I/O: 50ohms (else 25ohm)
        ori     r4, r4, 0x40            // CPU I/O: 40ohms (else 20ohm)
//      ori     r4, r4, 0x30            // Mem I/O   8 ohms
//      ori     r4, r4, 0x20            // Mem I/O: 13 ohms
        ori     r4, r4, 0x10            // Mem I/O: 20 ohms
//      ori     r4, r4, 0x00            // Mem I/O: 40 ohms
//      ori     r4, r4, 0x0c            // PCIClk:  8 ohms
//      ori     r4, r4, 0x08            // PCIClk: 13 ohms
//      ori     r4, r4, 0x04            // PCIClk: 20 ohms
        ori     r4, r4, 0x00            // PCIClk: 40 ohms
//      ori     r4, r4, 0x03            // MemClk: 8 ohms
//      ori     r4, r4, 0x02            // MemClk: 13.3 ohms
        ori     r4, r4, 0x01            // MemClk: 20 ohms
//      ori     r4, r4, 0x00            // MemClk: 40 ohms
#endif
        stb     r4, 3(r6)               /* ODCR is at +3 offset */

        /*-------MBEN */
        LOADPTR (r3, MEM_EN_ADR)        /* Set MBEN (a0) */
        stwbrx  r3,0,r5
        li      r4, 0x03                /* Enable bank 0 and 1 */
        stb     r4, 0(r6)               /* MBEN is at +0 offset */

        /*-------PGMAX */
        LOADPTR (r3, PAGE_MODE)
        stwbrx  r3,0,r5
        li      r4, 0x32
        stb     r4, 3(r6)               /* PAGE_MODE is at +3 offset */

        /* Wait before initializing other registers */

        lis     r4,0x0001
        mtctr   r4

KahluaX4wait200us:
        bdnz    KahluaX4wait200us

        /* Set MEMGO bit */

        LOADPTR (r3, MEM_CONT1_ADR)     /* MCCR1 (F0) |= PGMAX */
        stwbrx  r3,0,r5
        lwbrx   r4,0,r6                 /* old MCCR1 */
        oris    r4,r4,0x0008            /* MEMGO=1 */
        stwbrx  r4, 0, r6

        /* Wait again */

        addis   r4,r0,0x0002
        ori     r4,r4,0xffff

        mtctr   r4

KahluaX4wait8ref:
         bdnz    KahluaX4wait8ref

        sync
        eieio

        LOADPTR (r3, EUMBBAR)   /* Set EUMBAR Base */
        stwbrx  r3,0,r5
        LOADPTR(r4, EUMBBAR_VAL)
        stwbrx  r4,0,r6


        /*
         * Calculate absolute address in FLASH and jump there
         *-------------------------------------------------------------------
         */
        lis     r3, CFG_MONITOR_BASE@h
        ori     r3, r3, CFG_MONITOR_BASE@l
        addi    r3, r3, in_flash - _start + EXC_OFF_SYS_RESET
        mtlr    r3
        blr



/*
 *  void SIO_OUT(int char);
 *
 *  Configure Mousse COM1 to 9600 baud and write a character.
 *  Should be usable basically at any time.
 *
 *  r3 = character to output
 */
        .global SIO_OUT
SIO_OUT:
        /* r4 = intLock() */
        mfmsr   r4
        rlwinm  r5,r4,0,17,15
        mtmsr   r5
        isync

        /* r5 = serial port address */
        LOADPTR(r5, 0xffe08080)

        /* Init serial port */
        li      r6, 0x83
        stb     r6, 3(r5)
        li      r6, 0x78
        stb     r6, 0(r5)
        li      r6, 0x00
        stb     r6, 1(r5)
        li      r6, 0x03
        stb     r6, 3(r5)
        li      r6, 0x0b
        stb     r6, 4(r5)

        /* Wait for transmit buffer available */
sowait:
        lbz     r6, 5(r5)
        andi.   r0, r6, 0x40
        bc      12, 2, sowait        
 
        /* Transmit byte */
        stb     r3, 0(r5)

        /* intUnlock(r4) */
        rlwinm  r4,r4,0,16,16
        mfmsr   r5
        or      r4,r4,r5
        mtmsr   r4
        isync

        blr
#endif   /* CONFIG_MOUSSE */

/*
 * This code finishes saving the registers to the exception frame
 * and jumps to the appropriate handler for the exception.
 * Register r21 is pointer into trap frame, r1 has new stack pointer.
 */
	.globl	transfer_to_handler
transfer_to_handler:
	stw	r22,_NIP(r21)
	lis	r22,MSR_POW@h
	andc	r23,r23,r22
	stw	r23,_MSR(r21)
	SAVE_GPR(7, r21)
	SAVE_4GPRS(8, r21)
	SAVE_8GPRS(12, r21)
	SAVE_8GPRS(24, r21)
#if 0
	andi.	r23,r23,MSR_PR
	mfspr	r23,SPRG3		/* if from user, fix up tss.regs */
	beq	2f
	addi	r24,r1,STACK_FRAME_OVERHEAD
	stw	r24,PT_REGS(r23)
2:	addi	r2,r23,-TSS		/* set r2 to current */
	tovirt(r2,r2,r23)
#endif
	mflr	r23
	andi.	r24,r23,0x3f00		/* get vector offset */
	stw	r24,TRAP(r21)
	li	r22,0
	stw	r22,RESULT(r21)
	mtspr	SPRG2,r22		/* r1 is now kernel sp */
#if 0
	addi	r24,r2,TASK_STRUCT_SIZE /* check for kernel stack overflow */
	cmplw	0,r1,r2
	cmplw	1,r1,r24
	crand	1,1,4
	bgt	stack_ovf		/* if r2 < r1 < r2+TASK_STRUCT_SIZE */
#endif
	lwz	r24,0(r23)		/* virtual address of handler */
	lwz	r23,4(r23)		/* where to go when done */
	mtspr	SRR0,r24
	ori	r20,r20,0x30		/* enable IR, DR */
	mtspr	SRR1,r20
	mtlr	r23
	SYNC
	rfi				/* jump to handler, enable MMU */

int_return:
	mfmsr	r29		/* Disable interrupts */
	li	r4,0
	ori	r4,r4,MSR_EE
	andc	r29,r29,r4
	SYNC			/* Some chip revs need this... */
	mtmsr	r29
	SYNC
	lwz	r2,_CTR(r1)
	lwz	r0,_LINK(r1)
	mtctr	r2
	mtlr	r0
	lwz	r2,_XER(r1)
	lwz	r0,_CCR(r1)
	mtspr	XER,r2
	mtcrf	0xFF,r0
	REST_10GPRS(3, r1)
	REST_10GPRS(13, r1)
	REST_8GPRS(23, r1)
	REST_GPR(31, r1)
	lwz	r2,_NIP(r1)	/* Restore environment */
	lwz	r0,_MSR(r1)
	mtspr	SRR0,r2
	mtspr	SRR1,r0
	lwz	r0,GPR0(r1)
	lwz	r2,GPR2(r1)
	lwz	r1,GPR1(r1)
	SYNC
	rfi

/* Cache functions.
*/
	.globl	icache_enable
icache_enable:
	mfspr	r5,HID0		/* turn on the I cache. */
	ori	r5,r5,0x8800	/* Instruction cache only! */
	addis	r6,0,0xFFFF
	ori	r6,r6,0xF7FF
	and	r6,r5,r6	/* clear the invalidate bit */
	sync
	mtspr	HID0,r5
	mtspr	HID0,r6
	isync
	sync
	blr

	.globl	icache_disable
icache_disable:
	mfspr	r5,HID0
	addis	r6,0,0xFFFF
	ori	r6,r6,0x7FFF
	and	r5,r5,r6
	sync
	mtspr	HID0,r5
	isync
	sync
	blr

	.globl	icache_status
icache_status:
	mfspr	r3, HID0
	srwi	r3, r3, 15	/* >>15 & 1=> select bit 16 */
	andi.	r3, r3, 1
	blr

	.globl	dcache_enable
dcache_enable:
	mfspr	r5,HID0		/* turn on the D cache. */
	ori	r5,r5,0x4400	/* Data cache only! */
	mfspr	r4, PVR		/* read PVR */
	srawi	r3, r4, 16	/* shift off the least 16 bits */
	cmpi	0, 0, r3, 0xC	/* Check for Max pvr */
	bne	NotMax
	ori	r5,r5,0x0040	/* setting the DCFA bit, for Max rev 1 errata */
NotMax:
	addis	r6,0,0xFFFF
	ori	r6,r6,0xFBFF
	and	r6,r5,r6	/* clear the invalidate bit */
	sync
	mtspr	HID0,r5
	mtspr	HID0,r6
	isync
	sync
	blr

	.globl	dcache_disable
dcache_disable:
	mfspr	r5,HID0
	addis	r6,0,0xFFFF
	ori	r6,r6,0xBFFF
	and	r5,r5,r6
	sync
	mtspr	HID0,r5
	isync
	sync
	blr

	.globl	dcache_status
dcache_status:
	mfspr	r3, HID0
	srwi	r3, r3, 14	/* >>14 & 1=> select bit 17 */
	andi.	r3, r3, 1
	blr

	.globl	dc_read
dc_read:
/*TODO : who uses this, what should it do?
*/
	blr


	.globl get_pvr
get_pvr:
	mfspr	r3, PVR
	blr


#if defined(CONFIG_MOUSSE)
/*
 * Initialize the MMU using BAT entries and hardwired TLB
 * This obviates the need for any code in cpu_init_f which
 * configures the BAT registers.
*/
#define MEMORY_MGMT_MSR_BITS (MSR_DR | MSR_IR) /* Data and Inst Relocate */
        .global iommu_setup
        /* Initialize IO/MMU mappings via BAT method Ch. 7,
         * PPC Programming Reference
         */
iommu_setup:

/* initialize the BAT registers (SPRs 528 - 543 */
#define mtibat0u(x)     mtspr   528,(x)                 /* SPR 528 (IBAT0U) */
#define mtibat0l(x)     mtspr   529,(x)                 /* SPR 529 (IBAT0L) */
#define mtibat1u(x)     mtspr   530,(x)                 /* SPR 530 (IBAT1U) */
#define mtibat1l(x)     mtspr   531,(x)                 /* SPR 531 (IBAT1L) */
#define mtibat2u(x)     mtspr   532,(x)                 /* SPR 532 (IBAT2U) */
#define mtibat2l(x)     mtspr   533,(x)                 /* SPR 533 (IBAT2L) */
#define mtibat3u(x)     mtspr   534,(x)                 /* SPR 534 (IBAT3U) */
#define mtibat3l(x)     mtspr   535,(x)                 /* SPR 535 (IBAT3L) */
#define mtdbat0u(x)     mtspr   536,(x)                 /* SPR 536 (DBAT0U) */
#define mtdbat0l(x)     mtspr   537,(x)                 /* SPR 537 (DBAT0L) */
#define mtdbat1u(x)     mtspr   538,(x)                 /* SPR 538 (DBAT1U) */
#define mtdbat1l(x)     mtspr   539,(x)                 /* SPR 539 (DBAT1L) */
#define mtdbat2u(x)     mtspr   540,(x)                 /* SPR 540 (DBAT2U) */
#define mtdbat2l(x)     mtspr   541,(x)                 /* SPR 541 (DBAT2L) */
#define mtdbat3u(x)     mtspr   542,(x)                 /* SPR 542 (DBAT3U) */
#define mtdbat3l(x)     mtspr   543,(x)                 /* SPR 543 (DBAT3L) */


/* PowerPC processors do not necessarily initialize the BAT
   registers on power-up or reset.  So they are in an unknown
   state.  Before programming the BATs for the first time, all
   BAT registers MUST have their Vs and Vp bits cleared in the
   upper BAT half in order to avoid possibly having 2 BATs
   valid and mapping the same memory region.

   The reason for this is that, even with address translation
   disabled, multiple BAT hits for an address are treated as
   programming errors and can cause unpredictable results.

   It is up to the software to make sure it never has 2 IBAT
   mappings or 2 DBAT mappings that are valid for the same
   addresses.  It is not necessary to perform this code
   sequence every time the BATs are programmed, only when
   there is a possibility that there may be overlapping BAT
   entries.
   When programming the BATs in non-reset scenarios, even if
   you are sure that your new mapping will not temporarily
   create overlapping regions, it is still a wise idea to
   invalidate a BAT entry by setting its upper BAT register to
   all 0's before programming it.  This will avoid having a
   BAT marked valid that is in an unknown or transient state
*/

        addis   r5,0,0x0000
        mtibat0u(r5)
        mtibat0l(r5)
        mtibat1u(r5)
        mtibat1l(r5)
        mtibat2u(r5)
        mtibat2l(r5)
        mtibat3u(r5)
        mtibat3l(r5)
        mtdbat0u(r5)
        mtdbat0l(r5)
        mtdbat1u(r5)
        mtdbat1l(r5)
        mtdbat2u(r5)
        mtdbat2l(r5)
        mtdbat3u(r5)
        mtdbat3l(r5)
        isync

/*
 *  Set up I/D BAT0
 */
        lis     r4, CFG_DBAT0L@h
        ori     r4, r4, CFG_DBAT0L@l
        lis     r3, CFG_DBAT0U@h
        ori     r3, r3, CFG_DBAT0U@l

        mtdbat0l(r4)
        isync
        mtdbat0u(r3)
        isync
        sync

        lis     r4, CFG_IBAT0L@h
        ori     r4, r4, CFG_IBAT0L@l
        lis     r3, CFG_IBAT0U@h
        ori     r3, r3, CFG_IBAT0U@l

        isync
        mtibat0l(r4)
        isync
        mtibat0u(r3)
        isync
/*
 *  Set up I/D BAT1
 */
        lis     r4, CFG_IBAT1L@h
        ori     r4, r4, CFG_IBAT1L@l
        lis     r3, CFG_IBAT1U@h
        ori     r3, r3, CFG_IBAT1U@l

        isync
        mtibat1l(r4)
        isync
        mtibat1u(r3)
        isync
        mtdbat1l(r4)
        isync
        mtdbat1u(r3)
        isync
        sync

/*
 *  Set up I/D BAT2
 */
        lis     r4, CFG_IBAT2L@h
        ori     r4, r4, CFG_IBAT2L@l
        lis     r3, CFG_IBAT2U@h
        ori     r3, r3, CFG_IBAT2U@l

        isync
        mtibat2l(r4)
        isync
        mtibat2u(r3)
        isync
        mtdbat2l(r4)
        isync
        mtdbat2u(r3)
        isync
        sync

/*
 *  Setup I/D BAT3
 */
        lis     r4, CFG_IBAT3L@h
        ori     r4, r4, CFG_IBAT3L@l
        lis     r3, CFG_IBAT3U@h
        ori     r3, r3, CFG_IBAT3U@l

        isync
        mtibat3l(r4)
        isync
        mtibat3u(r3)
        isync
        mtdbat3l(r4)
        isync
        mtdbat3u(r3)
        isync
        sync


/*
 *  Invalidate all 64 TLB's
 */
        lis     r3, 0
        mtctr   r3
        lis     r5, 4

tlblp:
        tlbie   r3
        sync
        addi    r3, r3, 0x1000
        cmplw   r3, r5
        blt     tlblp

        sync

/*
 *  Enable Data Translation
 */
        lis     r4, MEMORY_MGMT_MSR_BITS@h
        ori     r4, r4, MEMORY_MGMT_MSR_BITS@l
        mfmsr   r3
        or      r3, r4, r3
        mtmsr   r3
        isync
        sync

        blr
#endif /* CONFIG_MOUSSE */


/*------------------------------------------------------------------------------*/

/*
 * void relocate_code (addr_sp, bd, addr_moni)
 *
 * This "function" does not return, instead it continues in RAM
 * after relocating the monitor code.
 *
 * r3 = dest
 * r4 = src
 * r5 = length in bytes
 * r6 = cachelinesize
 */
	.globl	relocate_code
relocate_code:

	mr	r1,  r3		/* Set new stack pointer		*/
	mr	r9,  r4		/* Save copy of Board Info pointer	*/
	mr	r10, r5		/* Save copy of Destination Address	*/

	mr	r3,  r5				/* Destination Address	*/
#ifdef DEBUG
	lis	r4, CFG_SDRAM_BASE@h		/* Source      Address	*/
	ori	r4, r4, CFG_SDRAM_BASE@l
#else
	lis	r4, CFG_MONITOR_BASE@h		/* Source      Address	*/
	ori	r4, r4, CFG_MONITOR_BASE@l
#endif
	lis	r5, CFG_MONITOR_LEN@h		/* Length in Bytes	*/
	ori	r5, r5, CFG_MONITOR_LEN@l
	li	r6, CFG_CACHELINE_SIZE		/* Cache Line Size	*/

	/*
	 * Fix GOT pointer:
	 *
	 * New GOT-PTR = (old GOT-PTR - CFG_MONITOR_BASE) + Destination Address
	 *
	 * Offset:
	 */
	sub	r15, r10, r4

	/* First our own GOT */
	add	r14, r14, r15
	/* the the one used by the C code */
	add	r30, r30, r15

	/*
	 * Now relocate code
	 */

	cmplw	cr1,r3,r4
	addi	r0,r5,3
	srwi.	r0,r0,2
	beq	cr1,4f		/* In place copy is not necessary	*/
	beq	7f		/* Protect against 0 count		*/
	mtctr	r0
	bge	cr1,2f

	la	r8,-4(r4)
	la	r7,-4(r3)
1:	lwzu	r0,4(r8)
	stwu	r0,4(r7)
	bdnz	1b
	b	4f

2:	slwi	r0,r0,2
	add	r8,r4,r0
	add	r7,r3,r0
3:	lwzu	r0,-4(r8)
	stwu	r0,-4(r7)
	bdnz	3b

/*
 * Now flush the cache: note that we must start from a cache aligned
 * address. Otherwise we might miss one cache line.
 */
4:	cmpwi	r6,0
	add	r5,r3,r5
	beq	7f		/* Always flush prefetch queue in any case */
	subi	r0,r6,1
	andc	r3,r3,r0
	mr	r4,r3
5:	dcbst	0,r4
	add	r4,r4,r6
	cmplw	r4,r5
	blt	5b
	sync			/* Wait for all dcbst to complete on bus */
	mr	r4,r3
6:	icbi	0,r4
	add	r4,r4,r6
	cmplw	r4,r5
	blt	6b
7:	sync			/* Wait for all icbi to complete on bus	*/
	isync

/*
 * We are done. Do not return, instead branch to second part of board
 * initialization, now running from RAM.
 */

	addi	r0, r10, in_ram - _start + EXC_OFF_SYS_RESET
	mtlr	r0
	blr

in_ram:

	/*
	 * Relocation Function, r14 point to got2+0x8000
	 *
	 * Adjust got2 pointers, no need to check for 0, this code
	 * already puts a few entries in the table.
	 */
	li	r0,__got2_entries@sectoff@l
	la	r3,GOT(_GOT2_TABLE_)
	lwz	r11,GOT(_GOT2_TABLE_)
	mtctr	r0
	sub	r11,r3,r11
	addi	r3,r3,-4
1:	lwzu	r0,4(r3)
	add	r0,r0,r11
	stw	r0,0(r3)
	bdnz	1b

	/*
	 * Now adjust the fixups and the pointers to the fixups
	 * in case we need to move ourselves again.
	 */
2:	li	r0,__fixup_entries@sectoff@l
	lwz	r3,GOT(_FIXUP_TABLE_)
	cmpwi	r0,0
	mtctr	r0
	addi	r3,r3,-4
	beq	4f
3:	lwzu	r4,4(r3)
	lwzux	r0,r4,r11
	add	r0,r0,r11
	stw	r10,0(r3)
	stw	r0,0(r4)
	bdnz	3b
4:
clear_bss:
	/*
	 * Now clear BSS segment
	 */
	lwz	r3,GOT(.bss)
	lwz	r4,GOT(_end)

	cmplw	0, r3, r4
	beq	6f

	li	r0, 0
5:
	stw	r0, 0(r3)
	addi	r3, r3, 4
	cmplw	0, r3, r4
	blt	5b
6:

	mr	r3, r9		/* Board Info pointer		*/
	mr	r4, r10		/* Destination Address		*/
	bl	board_init_r

	/* Problems accessing "end" in C, so do it here */
	.globl	get_endaddr
get_endaddr:
	lwz	r3,GOT(_end)
	blr

	/*
	 * Copy exception vector code to low memory
	 *
	 * r3: dest_addr
	 * r7: source address, r8: end address, r9: target address
	 */
	.globl	trap_init
trap_init:
	lwz	r7, GOT(_start)
	lwz	r8, GOT(_end_of_vectors)

	rlwinm	r9, r7, 0, 18, 31	/* _start & 0x3FFF	*/

	cmplw	0, r7, r8
	bgelr				/* return if r7>=r8 - just in case */

	mflr	r4			/* save link register		*/
1:
	lwz	r0, 0(r7)
	stw	r0, 0(r9)
	addi	r7, r7, 4
	addi	r9, r9, 4
	cmplw	0, r7, r8
	bne	1b

	/*
	 * relocate `hdlr' and `int_return' entries
	 */
	li	r7, .L_MachineCheck - _start + EXC_OFF_SYS_RESET
	li	r8, Alignment - _start + EXC_OFF_SYS_RESET
2:
	bl	trap_reloc
	addi	r7, r7, 0x100		/* next exception vector	*/
	cmplw	0, r7, r8
	blt	2b

	li	r7, .L_Alignment - _start + EXC_OFF_SYS_RESET
	bl	trap_reloc

	li	r7, .L_ProgramCheck - _start + EXC_OFF_SYS_RESET
	bl	trap_reloc

	li	r7, .L_FPUnavailable - _start + EXC_OFF_SYS_RESET
	li	r8, SystemCall - _start + EXC_OFF_SYS_RESET
3:
	bl	trap_reloc
	addi	r7, r7, 0x100		/* next exception vector	*/
	cmplw	0, r7, r8
	blt	3b

	li	r7, .L_SingleStep - _start + EXC_OFF_SYS_RESET
	li	r8, _end_of_vectors - _start + EXC_OFF_SYS_RESET
4:
	bl	trap_reloc
	addi	r7, r7, 0x100		/* next exception vector	*/
	cmplw	0, r7, r8
	blt	4b

	mtlr	r4			/* restore link register	*/
	blr

	/*
	 * Function: relocate entries for one exception vector
	 */
trap_reloc:
	lwz	r0, 0(r7)		/* hdlr ...			*/
	add	r0, r0, r3		/*  ... += dest_addr		*/
	stw	r0, 0(r7)

	lwz	r0, 4(r7)		/* int_return ...		*/
	add	r0, r0, r3		/*  ... += dest_addr		*/
	stw	r0, 4(r7)

	blr

	/* Setup the BAT registers.
	 */
setup_bats:
	lis	r4, CFG_IBAT0L@h
	ori	r4, r4, CFG_IBAT0L@l
	lis	r3, CFG_IBAT0U@h
	ori	r3, r3, CFG_IBAT0U@l
	mtspr	IBAT0L, r4
	mtspr	IBAT0U, r3
	isync

	lis	r4, CFG_DBAT0L@h
	ori	r4, r4, CFG_DBAT0L@l
	lis	r3, CFG_DBAT0U@h
	ori	r3, r3, CFG_DBAT0U@l
	mtspr	DBAT0L, r4
	mtspr	DBAT0U, r3
	isync

	lis	r4, CFG_IBAT1L@h
	ori	r4, r4, CFG_IBAT1L@l
	lis	r3, CFG_IBAT1U@h
	ori	r3, r3, CFG_IBAT1U@l
	mtspr	IBAT1L, r4
	mtspr	IBAT1U, r3
	isync

	lis	r4, CFG_DBAT1L@h
	ori	r4, r4, CFG_DBAT1L@l
	lis	r3, CFG_DBAT1U@h
	ori	r3, r3, CFG_DBAT1U@l
	mtspr	DBAT1L, r4
	mtspr	DBAT1U, r3
	isync

	lis	r4, CFG_IBAT2L@h
	ori	r4, r4, CFG_IBAT2L@l
	lis	r3, CFG_IBAT2U@h
	ori	r3, r3, CFG_IBAT2U@l
	mtspr	IBAT2L, r4
	mtspr	IBAT2U, r3
	isync

	lis	r4, CFG_DBAT2L@h
	ori	r4, r4, CFG_DBAT2L@l
	lis	r3, CFG_DBAT2U@h
	ori	r3, r3, CFG_DBAT2U@l
	mtspr	DBAT2L, r4
	mtspr	DBAT2U, r3
	isync

	lis	r4, CFG_IBAT3L@h
	ori	r4, r4, CFG_IBAT3L@l
	lis	r3, CFG_IBAT3U@h
	ori	r3, r3, CFG_IBAT3U@l
	mtspr	IBAT3L, r4
	mtspr	IBAT3U, r3
	isync

	lis	r4, CFG_DBAT3L@h
	ori	r4, r4, CFG_DBAT3L@l
	lis	r3, CFG_DBAT3U@h
	ori	r3, r3, CFG_DBAT3U@l
	mtspr	DBAT3L, r4
	mtspr	DBAT3U, r3
	isync

	/* Invalidate TLBs.
	 * -> for (val = 0; val < 0x20000; val+=0x1000)
	 * ->   tlbie(val);
	 */
	lis	r3, 0
	lis	r5, 2

1:
	tlbie	r3
	addi	r3, r3, 0x1000
	cmp	0, 0, r3, r5
	blt	1b

	blr


