/*
 * (C) Copyright 2000, 2001
 * John W. Linville, LVL7 Systems, Inc., linville@lvl7.com.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 *
 * Configuration settings for the BMW board.
 *
 */

/* ------------------------------------------------------------------------- */

#ifndef __CONFIG_H
#define __CONFIG_H

/*
 * High Level Configuration Options
 */

#define CONFIG_BMW              1

#define CONFIG_BOARD_PRE_INIT   1

#define CONFIG_MPC824x          1
#define CONFIG_MPC8245          1
#define CFG_ADDR_MAP_B          1

#define CONFIG_CONS_INDEX       1
#define CONFIG_BAUDRATE         9600

#define CONFIG_BOOTDELAY        5
#define CONFIG_BOOTCOMMAND      "cp.b ffe00000 00400000 100000 ; " \
                                "cp.b fff30000 00500000 d0000 ; " \
                                "bootm 00400000"
#define CONFIG_BOOTARGS         "root=/dev/ram"

#define CONFIG_EXTRA_ENV_SETTINGS "nfsprep=setenv bootargs root=/dev/nfs " \
                                  "ip=$ipaddr:$serverip:$gatewayip:$netmask ; "\
                                  "setenv bootcmd tftpboot \\; bootm\0" \
                                  "flashprep=setenv bootargs root=/dev/ram ; " \
                                  "setenv bootcmd " \
                                  "cp.b ffe00000 00400000 100000 \\; " \
                                  "cp.b fff30000 00500000 d0000 \\; " \
                                  "bootm 00400000\0"

#define CONFIG_COMMANDS       ((CONFIG_CMD_DFL | \
                                CFG_CMD_PCI | \
                                CFG_CMD_ENV | \
                                CFG_CMD_FLASH | \
                                CFG_CMD_ASKENV | \
                                CFG_CMD_DATE | \
                                CFG_CMD_ELF))

/* This must be included AFTER the definition of CONFIG_COMMANDS (if any)
 */
#include <cmd_confdefs.h>

/*
 * Miscellaneous configurable options
 */
#define CFG_LONGHELP          1     /* undef to save memory     */
#define CFG_PROMPT            "=>"  /* Monitor Command Prompt   */

#define CFG_HUSH_PARSER
#ifdef  CFG_HUSH_PARSER
#define CFG_PROMPT_HUSH_PS2   "> "
#endif

#define CFG_CBSIZE            1024   /* Console I/O Buffer Size  */
#define CFG_PBSIZE            (CFG_CBSIZE + sizeof(CFG_PROMPT) + 16)
#define CFG_MAXARGS           16     /* Max number of command args   */

#define CFG_BARGSIZE          CFG_CBSIZE  /* Boot Argument Buffer Size    */
#define CFG_LOAD_ADDR         0x00100000  /* Default load address         */

#define CFG_HZ                1000   /* decrementer freq: 1 ms */

/*-----------------------------------------------------------------------
 * Start addresses for the final memory configuration
 * (Set up by the startup code)
 * Please note that CFG_SDRAM_BASE _must_ start at 0
 */
#define CFG_SDRAM_BASE      0x00000000
#define CFG_MAX_RAM_SIZE    0x04000000

#define CFG_MONITOR_BASE    TEXT_BASE

#ifdef DEBUG
#define CFG_MONITOR_LEN     (4 << 20)   /* lots of mem ... */
#else
#define CFG_MONITOR_LEN     (192 << 10) /* PLCC bootrom (reserve 192 K) */
#endif
#define CFG_MALLOC_LEN      (2*(4096 << 10))    /* 2*4096kB for malloc()  */

#define CFG_MEMTEST_START   0x00004000  /* memtest works on      */
#define CFG_MEMTEST_END     0x04000000  /* 0 ... 64 MB in DRAM   */

#define CFG_EUMB_ADDR       0xFC000000

#define CFG_FLASH_BASE      0xFFE00000
#define CFG_FLASH_SIZE      ((uint)(2 * (1024 * 1024)))

#define CFG_RESET_ADDRESS   0xFFF00100

#define CFG_BAUDRATE_TABLE  { 9600, 19200, 38400, 57600, 115200 }

/*
 * NS16550 Configuration
 */
#define CFG_NS16550
#define CFG_NS16550_SERIAL

#define CFG_NS16550_REG_SIZE    1

#define CFG_NS16550_CLK         100000000

#define CFG_NS16550_COM1        (CFG_EUMB_ADDR + 0x4500)
#define CFG_NS16550_COM2        (CFG_EUMB_ADDR + 0x4600)

/*-----------------------------------------------------------------------
 * Definitions for initial stack pointer and data area
 */
#define CFG_INIT_RAM_ADDR    0x01fff000
#define CFG_INIT_RAM_END     0x1000  /* End of used area */
#define CFG_INIT_DATA_SIZE   128 /* size in bytes reserved for initial data */
#define CFG_INIT_DATA_OFFSET (CFG_INIT_RAM_END - CFG_INIT_DATA_SIZE)
#define CFG_INIT_SP_OFFSET   CFG_INIT_DATA_OFFSET

/*
 * Low Level Configuration Settings
 * (address mappings, register initial values, etc.)
 * You should know what you are doing if you make changes here.
 * For the detail description refer to the MPC8245 user's manual.
 */

/* external frequency to pll -- detect at runtime w/ RTC */
#define CONFIG_SYS_CLK_FREQ  *((unsigned long *)0x02000000)

#define CFG_ROMNAL      8
#define CFG_ROMFAL      11

#define CFG_REFINT      0x3a5
#define CFG_ASRISE      8
#define CFG_ASFALL      4

/* the following are for SDRAM only*/
#define CFG_BSTOPRE     0x25c  /*
                                *NOTE: This value is confusing to say
                                *the least.  Re-check to ensure correctness
                                *as its bits are split up amongst various
                                *parts of MCCR3 and MCCR4
                                */

#define CFG_REFREC              8

#define CFG_RDLAT               4       /* data latency from read command */

#define CFG_PRETOACT            3       /* Precharge to activate interval */
#define CFG_ACTTOPRE            5       /* Activate to Precharge interval */
#define CFG_ACTORW              3       /* Activate to R/W */
#define CFG_SDMODE_CAS_LAT      2       /* SDMODE CAS latency */
#define CFG_SDMODE_WRAP         0       /* SDMODE wrap type */

#define CFG_REGISTERD_TYPE_BUFFER  1
#define CFG_EXTROM                 1
#define CFG_REGDIMM                0

/* memory bank settings*/
/*
 * only bits 20-29 are actually used from these vales to set the
 * start/end address the upper two bits will be 0, and the lower 20
 * bits will be set to 0x00000 for a start address, or 0xfffff for an
 * end address
 */
#define CFG_BANK0_START         0x00000000
#define CFG_BANK0_END           (CFG_MAX_RAM_SIZE - 1)
#define CFG_BANK0_ENABLE        1
#define CFG_BANK1_START         0x3ff00000
#define CFG_BANK1_END           0x3ff00000
#define CFG_BANK1_ENABLE        0
#define CFG_BANK2_START         0x3ff00000
#define CFG_BANK2_END           0x3ff00000
#define CFG_BANK2_ENABLE        0
#define CFG_BANK3_START         0x3ff00000
#define CFG_BANK3_END           0x3ff00000
#define CFG_BANK3_ENABLE        0
#define CFG_BANK4_START         0x3ff00000
#define CFG_BANK4_END           0x3ff00000
#define CFG_BANK4_ENABLE        0
#define CFG_BANK5_START         0x3ff00000
#define CFG_BANK5_END           0x3ff00000
#define CFG_BANK5_ENABLE        0
#define CFG_BANK6_START         0x3ff00000
#define CFG_BANK6_END           0x3ff00000
#define CFG_BANK6_ENABLE        0
#define CFG_BANK7_START         0x3ff00000
#define CFG_BANK7_END           0x3ff00000
#define CFG_BANK7_ENABLE        0

/*
 * Memory bank enable bitmask, specifying which of the banks defined above
 * are actually present. MSB is for bank #7, LSB is for bank #0.
 */
#define CFG_BANK_ENABLE         0x01

#define CFG_ODCR                0x7f    /* configures line driver impedances,*/
                                        /* see 8245 book for bit definitions */
#define CFG_PGMAX               0xd0
                                        /* currently accessed page in memory */
                                        /* see 8245 book for details */

/* SDRAM 0 - 256MB */
#define CFG_IBAT0L      (CFG_SDRAM_BASE | BATL_PP_10 | BATL_MEMCOHERENCE)
#define CFG_IBAT0U      (CFG_SDRAM_BASE | BATU_BL_256M | BATU_VS | BATU_VP)

/* RCS2 & RCS3 */
#define CFG_IBAT1L      (0x70000000 | BATL_PP_10 | BATL_CACHEINHIBIT)
#define CFG_IBAT1U      (0x70000000 | BATU_BL_256M | BATU_VS | BATU_VP)

/* PCI memory */
#define CFG_IBAT2L      (0x80000000 | BATL_PP_10 | BATL_CACHEINHIBIT)
#define CFG_IBAT2U      (0x80000000 | BATU_BL_256M | BATU_VS | BATU_VP)

/* Flash, config addrs, etc */
#define CFG_IBAT3L      (0xF0000000 | BATL_PP_10 | BATL_CACHEINHIBIT)
#define CFG_IBAT3U      (0xF0000000 | BATU_BL_256M | BATU_VS | BATU_VP)

#define CFG_DBAT0L      CFG_IBAT0L
#define CFG_DBAT0U      CFG_IBAT0U
#define CFG_DBAT1L      CFG_IBAT1L
#define CFG_DBAT1U      CFG_IBAT1U
#define CFG_DBAT2L      CFG_IBAT2L
#define CFG_DBAT2U      CFG_IBAT2U
#define CFG_DBAT3L      CFG_IBAT3L
#define CFG_DBAT3U      CFG_IBAT3U

/*
 * For booting Linux, the board info and command line data
 * have to be in the first 8 MB of memory, since this is
 * the maximum mapped by the Linux kernel during initialization.
 */
#define CFG_BOOTMAPSZ           (8 << 20)    /* Initial Memory map for Linux */

/*-----------------------------------------------------------------------
 * FLASH organization
 */
#define CFG_MAX_FLASH_BANKS     1       /* Max number of flash banks         */
#define CFG_MAX_FLASH_SECT      35      /* Max number of sectors in one bank */

/*-----------------------------------------------------------------------
 * RTC/NVRAM
 */
#define CONFIG_RTC_DS174x            1 /*turn on the rtc*/
#define CFG_ENV_IS_IN_NVRAM          1
#define CFG_NVRAM_ACCESS_ROUTINE     1
#define CFG_NVRAM_SIZE               ((8 * 1024) - 8)
#define CFG_NVRAM_BASE_ADDR          0x7c004000
#define CFG_ENV_ADDR                 0x7c004800
#define CFG_ENV_SIZE                 0x0800

/*-----------------------------------------------------------------------
 * Cache Configuration
 */
#define CFG_CACHELINE_SIZE      32      /* For MPC8245 CPU */
#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#  define CFG_CACHELINE_SHIFT   5       /* log base 2 of the above value */
#endif


/*
 * Internal Definitions
 *
 * Boot Flags
 */
#define BOOTFLAG_COLD           0x01    /* Normal Power-On: Boot from FLASH */
#define BOOTFLAG_WARM           0x02    /* Software reboot */

/*-----------------------------------------------------------------------
 * PCI stuff
 */
#define CONFIG_PCI              1       /* include pci support          */
#define CONFIG_PCI_PNP          1

#endif  /* __CONFIG_H */
