/*
 * PPCBoot configuration file for LVL7's AmberJack boards.
 *
 * Copyright (c) 2002 LVL7 Systems, Inc., 13000 Weston Pkwy, Cary, NC 27513
 */

#ifndef __CONFIG_H
#define __CONFIG_H

#include <../board/lvl7/aj/aj.h>

#define CONFIG_MPC855		1
#define CONFIG_LVL7AJ		1

#define CONFIG_BOARD_PRE_INIT	1

#define CONFIG_8xx_GCLK_FREQ	50000000

#define CONFIG_8xx_CONS_SMC1	1
#define CONFIG_BAUDRATE		9600

#define CONFIG_BOOTDELAY	5
#define CONFIG_BOOTCOMMAND	"run diagscheck && run bootsequence"
#define CONFIG_BOOT_RETRY_TIME	-1
#define CONFIG_BOOT_RETRY_MIN	30

#define CONFIG_RTC_MPC8xx	1

#define CONFIG_AUTOBOOT_KEYED
#define CONFIG_AUTOBOOT_PROMPT	"autoboot in %d seconds (SPACE to stop)...\n"
#define CONFIG_AUTOBOOT_STOP_STR	" "

#define CONFIG_ZERO_BOOTDELAY_CHECK

#define CONFIG_RAMBOOTCOMMAND	"setenv bootargs root=/dev/ram ; " \
    "bootm fff60000 ff800000"

#define CONFIG_NFSBOOTCOMMAND	"setenv bootargs root=/dev/nfs " \
    "ip=$ipaddr:$serverip:$gatewayip:$netmask ; tftpboot"

#define CONFIG_EXTRA_ENV_SETTINGS "autostart=yes\0" \
                                  "nfsprep=setenv bootsequence run nfsboot ; " \
                                  "saveenv\0" \
                                  "ramprep=setenv bootsequence run ramboot ; " \
                                  "saveenv\0" \
                                  "vpdaddr=0003ff80\0" \
                                  "diagscmd=eeprom read 50 $vpdaddr 0 80 ; " \
                                  "bootm ffff0000 $vpdaddr\0" \
                                  "diags=yes\0" \
                                  "diagscheck=if printenv diags ; " \
                                  "then run diagscmd ; " \
                                  "else echo Diagnostics disabled. ; fi\0" \
                                  "bootcmd run diagscheck && " \
                                  "run bootsequence\0" \
                                  "bootsequence=run ramboot\0" \
                                  "bootfile=aj_kernel.image\0"

#define CONFIG_COMMANDS 	(CONFIG_CMD_DFL | \
				 CFG_CMD_DHCP | \
				 CFG_CMD_REGINFO | \
				 CFG_CMD_IMMAP | \
				 CFG_CMD_I2C | \
				 CFG_CMD_EEPROM | \
				 CFG_CMD_JFFS2 | \
				 CFG_CMD_SDRAM | \
				 CFG_CMD_MII | \
				 CFG_CMD_DATE | \
				 CFG_CMD_ELF)

/* this must be included AFTER the definition of CONFIG_COMMANDS (if any) */
#include <cmd_confdefs.h>

#undef CONFIG_WATCHDOG /* come-back later for this */

#define CONFIG_MII

#define CONFIG_I2C
#define CFG_I2C_SPEED	50000
#define CFG_I2C_SLAVE	0xFE

#define CFG_I2C_MULTI_EEPROMS
#define CFG_I2C_EEPROM_ADDR_LEN	1

#define CONFIG_HARD_I2C

#define FEC_ENET		1
#define CFG_DISCOVER_PHY

#define CFG_LONGHELP		1

#define CFG_PROMPT		"=>"

#define CFG_HUSH_PARSER
#ifdef  CFG_HUSH_PARSER
#define CFG_PROMPT_HUSH_PS2     "> "
#endif

#define CFG_CBSIZE		1024

#define CFG_PBSIZE 		(CFG_CBSIZE+sizeof(CFG_PROMPT)+16)

#define CFG_MAXARGS		16

#define CFG_BARGSIZE		CFG_CBSIZE
 
#define CFG_BAUDRATE_TABLE      { 9600, 19200, 38400, 57600, 115200 }

#define CFG_MEMTEST_START	0x0100000
#define CFG_MEMTEST_END		0x0400000
 
#define CFG_SDRAM_BASE		0x00000000
#define CFG_FLASH_BASE		0xff800000

#define CFG_MONITOR_BASE	TEXT_BASE
#define CFG_MONITOR_LEN		(256 << 10)

#define CFG_MALLOC_LEN		(384 << 10)

#define CFG_BOOTMAPSZ		(8 << 20)

#define CFG_MAX_FLASH_BANKS	4	/* max number of memory banks */
#define CFG_MAX_FLASH_SECT	71	/* max number of sectors on one chip */
#define CFG_FLASH_ERASE_TOUT	8000	/* flash erase timeout (in ms) */
#define CFG_FLASH_WRITE_TOUT	500	/* flash write timeout (in ms) */

#define CFG_ENV_IS_IN_FLASH	1

#define CFG_ENV_OFFSET		0x00730000
#define CFG_ENV_SECT_SIZE	0x00010000

#define CFG_CACHELINE_SIZE	16
#define CFG_IMMR		0xff000000
#define CFG_SIUMCR		0 /* ??? */
#define CFG_SYPCR		(SYPCR_SWTC | SYPCR_BMT | SYPCR_BME | SYPCR_SWF)
#define CFG_TBSCR		(TBSCR_TBF | TBSCR_TBE)
#define CFG_PISCR		PISCR_PITF
#define CFG_PLPRCR		0
#define CFG_DER			0 /* ??? */

#define CFG_MAMR		0

#define SCCR_MASK		0xffffffff
#define CFG_SCCR		0 /* ??? */

#define CFG_OR0_PRELIM		FLASH_OR0VALUE
#define CFG_BR0_PRELIM		FLASH_BR0VALUE

/*
 * Definitions for initial stack pointer and data area (in DPRAM)
 */
#define CFG_INIT_RAM_ADDR       CFG_IMMR
#define CFG_INIT_RAM_END        0x3000  /* End of used area in DPRAM */
#define CFG_INIT_DATA_SIZE      64  /* size in bytes rsrvd for initial data */
#define CFG_INIT_DATA_OFFSET    (CFG_INIT_RAM_END - CFG_INIT_DATA_SIZE)
#define CFG_INIT_SP_OFFSET      CFG_INIT_DATA_OFFSET

#define CFG_LOAD_ADDR		0x00100000
 
#define CFG_HZ			1000

/*-----------------------------------------------------------------------
 * Reset Address
 *
 * In order to reset the CPU, ppcboot jumps to a special address which
 * causes a machine check exception. The default address for this is
 * CFG_MONITOR_BASE - sizeof (ulong), which might not always work, eg. when
 * testing the monitor in RAM using a JTAG debugger.
 *
 * Just set CFG_RESET_ADDRESS to an address that you know is sure to
 * cause a bus error on your hardware.
 */
#define CFG_RESET_ADDRESS       0xC0000000

/*
 * JFFS2 Configuration
 */
#define CFG_JFFS2_FIRST_BANK	0
#define CFG_JFFS2_NUM_BANKS	4
#define CFG_JFFS2_FIRST_SECTOR	0

/*
 * Boot Flags
 */
#define BOOTFLAG_COLD   0x01            /* Normal Power-On: Boot from FLASH */
#define BOOTFLAG_WARM   0x02            /* Software reboot */

#endif /* __CONFIG_H */
