/*
 * (C) Copyright 2000
 * Rob Taylor, Flying Pig Systems. robt@flyingpig.com.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <mpc824x.h>
#include <pci.h>


/*
 *Defines for accessing I2C registers in eumb
 */
#define I2CADR    0x00003000
#define I2CFDR    0x00003004
#define I2CCR     0x00003008
#define I2CSR     0x0000300C
#define I2CDR     0x00003010

int checkboard (void)
{

	printf("Board: Jabil PSG 24-port GB Switch\n");

	return 0;
}

long int initdram (int board_type)
{
	int		 i, cnt;
	volatile uchar * base= CFG_SDRAM_BASE;
	volatile ulong * addr;
	ulong		 save[32];
	ulong		 val, ret  = 0;

	for (i=0, cnt=(CFG_MAX_RAM_SIZE / sizeof(long)) >> 1;
	     cnt > 0; cnt >>= 1) {
		addr = (volatile ulong *)base + cnt;
		save[i++] = *addr;
		*addr = ~cnt;
	}

	addr = (volatile ulong *)base;
	save[i] = *addr;
	*addr = 0;

	if (*addr != 0) {
		*addr = save[i];
		goto Done;
	}

	for (cnt = 1; cnt <= CFG_MAX_RAM_SIZE / sizeof(long); cnt <<= 1) {
		addr = (volatile ulong *)base + cnt;
		val = *addr;
		*addr = save[--i];
		if (val != ~cnt) {
			ret = cnt * sizeof(long);
			goto Done;
		}
	}

	ret = CFG_MAX_RAM_SIZE;
Done:
	return ret;
}

int board_pre_init(void)
{
	ulong val;
        unsigned int eumbbar = CFG_EUMB_ADDR;

	/* Configurations for RCS1 */
	CONFIG_READ_WORD(MCCR4, val);
	val |= 0x00020000;
	CONFIG_WRITE_WORD(MCCR4, val);

	/* Configurations for RCS2 and RCS3 */
	CONFIG_WRITE_WORD(0x800000d0, 0x84000000);
	CONFIG_WRITE_WORD(0x800000d4, 0x00000000);
	CONFIG_WRITE_WORD(0x800000d8, 0x0c000000);

	/*
	 * configure the duart to use our 4 serial lines
	 * as 2 uarts
	 */
        store_runtime_reg(eumbbar,0x4511,0x1);

	/*
	 * this is the I2C fixup algorithm suggested in
	 * 10.4.6 of the 8245 manual for when SDK is low
	 * and SCK needs to be driven low
	 */

	/* enable master, disable I2C */  
	store_runtime_reg(eumbbar,I2CCR,0x20);

	/* set the frequency divider */
	val = load_runtime_reg (eumbbar, I2CFDR);
	val = (val & 0xffffffc0) | 0x31;
	store_runtime_reg (eumbbar, I2CFDR, val);

	/* enable I2C */
	store_runtime_reg(eumbbar,I2CCR,0xA0);

	/* read I2CDR */
	val = load_runtime_reg(eumbbar,I2CDR);

	/* return I2C driver to slave mode */
	store_runtime_reg(eumbbar,I2CCR,0x80);
 
	return (0);
}

/*
 * Initialize PCI Devices, report devices found.
 */
#ifndef CONFIG_PCI_PNP
static struct pci_config_table pci_jabil_config_table[] = {
	{ }
};
#endif

struct pci_controller hose = {
#ifndef CONFIG_PCI_PNP
	config_table: pci_jabil_config_table,
#endif
};

void pci_init_board(void)
{
	pci_mpc824x_init(&hose);
}
